/*
 * Stellarium360 astronomy simulation and visualization
 *
 * Copyright (C) 2002 Fabien Chereau
 * Copyright (C) 2009 Digitalis Education Solutions, Inc.
 * Copyright (C) 2014 of the LSS Team & Association Sirius
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Stellarium360 is a free open project of of LSS team
 * See the TRADEMARKS file for free open project usage requirements.
 *
 */

#ifndef _ENTITYSUN_HPP_
#define _ENTITYSUN_HPP_

#include "program_object.hpp"
#include "object_base.hpp"
#include "entity.hpp"
#include "utility.hpp"
#include "s_font.hpp"
#include "tone_reproductor.hpp"
#include "vecmath.hpp"
#include "callbacks.hpp"
#include "fader.hpp"
#include "translator.hpp"
#include "orbit.hpp"
#include "model3D.hpp"
#include "ring.hpp"
#include <list>
#include <string>


using namespace std;



class EntitySun : public Entity
{
public:

	Sun(
	       int flag_lighting,
	       double radius,
	       double oblateness,
           Vec3f color,
           Vec3f label_color,
           Vec3f orbit_color,
           float _sol_local_day,
	       float albedo,
	       const string& tex_map_path,
	       const string& tex_map_name,
	       const string& tex_halo_name,
	       const string& model_name,
	       model3D_mgr * _myObjMgr,
	);
	~Sun();

	// Draw the Planet, if hint_ON is != 0 draw a circle and the name as well
	// Return the squared distance in pixels between the current and the
	// previous position this Entity was drawn at.
	virtual double draw(Projector* prj, const Navigator* nav, const ToneReproductor* eye,
	            int flag_point, bool stencil, bool depthTest, bool drawHomePlanet,
				bool selected);

protected:

	virtual void set_big_halo(const string& halotexfile, const string& path);

	// Draw the 3D sphere
	virtual void draw_sphere(const Projector* prj, const Navigator * nav, const Mat4d& mat, float screen_sz);

	// Draw the big halo (for sun or moon)
	virtual void draw_big_halo(const Navigator* nav, const Projector* prj, const ToneReproductor* eye);
};

#endif // _ENTITY_HPP_
