/*
 * Stellarium360 astronomy simulation and visualization
 *
 * Copyright (C) 2005 Robert Spearman
 * Copyright (C) 2009 Digitalis Education Solutions, Inc.
 * Copyright (C) 2013 of the LSS team
 * Copyright (C) 2014 of the LSS Team & Association Sirius
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Stellarium360 is a free open project of of LSS team
 * See the TRADEMARKS file for free open project usage requirements.
 *
 */

/* This class handles parsing of a simple command syntax for scripting,
   UI components, network commands, etc.
*/

#ifndef _APP_COMMAND_INTERFACE_H_
#define _APP_COMMAND_INTERFACE_H_

#include "core.hpp"
#include "command_interface.hpp"
#include "app.hpp"
#include "utility.hpp"
#include "script_mgr.hpp"
#include "audio.hpp"
#include "external_viewer.hpp"
//#include "message_mgr.hpp"

using namespace std;

// Predeclaration of the Core class
class Core;
class App;
//class Message_Mgr;

class AppCommandInterface : CommandInterface
{

public:
	AppCommandInterface(Core * core, App * app);
	virtual ~AppCommandInterface();
	virtual int execute_command(string commandline);
	virtual int execute_command(string command, double arg);
	virtual int execute_command(string command, int arg);
	virtual int execute_command(string command, unsigned long int &wait, bool trusted);
	virtual int set_flag(string name, string value, bool &newval, bool trusted);
	//~ void update(int delta_time);
	//~ void enableAudio();
	//~ void disableAudio();
	string getErrorString();

private:
	Core * stcore;
	App * stapp;
	//Audio * audio;  // for audio track from script
	ExternalViewer * ExtViewer; //! - external media viewer for scripting
	//bool audioDisabled; // - whether audio is disabled temporarily or not, master
	string debug_message;  //! for 'execute_command' error details
	//Message_Mgr *message;
};


#endif // _APP_COMMAND_INTERFACE_H
