/*
 * Stellarium360 astronomy simulation and visualization
 *
 * Author: Trystan Larey-Williams
 * Copyright (C) 2009 Digitalis Education Solutions, Inc.
 * Copyright (C) 2014 of the LSS Team & Association Sirius
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Stellarium360 is a free open project of of LSS team
 * See the TRADEMARKS file for free open project usage requirements.
 *
 */

#include "app_settings.hpp"
#include "stellarium360.hpp"
#include "utility.hpp"
#include "log.hpp"

// Stub implementations of functions that don't exist on windows and visa versa
#if defined(WIN32) || defined(CYGWIN) || defined(__MINGW32__) || defined(MINGW32)
	time_t timegm(struct tm*) {
		time_t t;
		memset( &t, 0, sizeof(t) );
		return t;
	}
#else
	int GetTimeZoneInformation( TIME_ZONE_INFORMATION* info ) {
		return 0;
	}
#endif 

AppSettings* AppSettings::m_instance = NULL;

AppSettings::AppSettings( string configDir, string dataRoot, string lDir ) : m_configDir(configDir),
																			 m_dataRoot(dataRoot),
																			 m_lDir(lDir) {
}

void AppSettings::Init( string configDir, string dataRoot, string lDir ) {
	if( m_instance )
		delete m_instance;
	m_instance = new AppSettings( configDir, dataRoot, lDir );
}

AppSettings* AppSettings::Instance(){
	if( !m_instance )
		return new AppSettings( "", "", "" );
	else
		return m_instance;
}

const bool AppSettings::OSX(void) const {
#if defined(MACOSX)
	return true;
#else
	return false;
#endif
}

const bool AppSettings::Windows(void) const {
	#if defined(WIN32) || defined(CYGWIN) || defined(__MINGW32__) || defined(MINGW32)
		return true;
	#else
		return false;
	#endif
}

const bool AppSettings::Unix(void) const {
	#if !defined(MACOSX) && !defined(WIN32) && !defined(CYGWIN) && !defined(__MINGW32__) && !defined(MINGW32)
		return true;
	#else
		return false;
	#endif
}

void AppSettings::loadAppSettings( InitParser* const conf ) const {
	conf->load(m_configDir + "config.ini");
}

const string AppSettings::getConfigDir(void) const {
	return m_configDir;
}

const string AppSettings::getConfigFile(void) const {
	return getConfigDir() + "config.ini";
}

const string AppSettings::getlDir(void) const {
	return m_lDir;
}

const string AppSettings::getDataRoot(void) const {
	return m_dataRoot;
}
