/*
 * Stellarium360 astronomy simulation and visualization
 *
 * Copyright (C) 2005 Robert Spearman
 * Copyright (C) 2009 Digitalis Education Solutions, Inc.
 * Copyright (C) 2014 of the LSS Team & Association Sirius
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Stellarium360 is a free open project of of LSS team
 * See the TRADEMARKS file for free open project usage requirements.
 *
 */

// manage an audio track

#ifndef _AUDIO_H_
#define _AUDIO_H_

#include <string>
#include "SDL2/SDL.h"
#include "SDL2/SDL_mixer.h"


#define MAX_CHANNELS 8


class Audio
{
	friend class Message_Mgr;

public:
	Audio();
	~Audio();

	//music
	void set_volumeMusic(int _value);
	void increment_volumeMusic(int value=5);
	void decrement_volumeMusic(int value=5);
	void loadMusic(std::string filename);
	void playMusic(bool loop);
	void pauseMusic();
	void resumeMusic();
	void rewindMusic();
	void haltMusic();
	void syncMusic();
	void update(int delta_time);
	void dropMusic();

	//sound
	void loadChunk(std::string filename);
	void playChunk();
	void pauseChunk();
	void resumeChunk();
	void haltChunk();
	void dropChunk(std::string filename);

private:
	float music_get_length(std::string filename);
	int master_volume;
	float music_length;
	Mix_Music *track = NULL;
	Mix_Chunk *sound[MAX_CHANNELS];
	std::string sound_name[MAX_CHANNELS];
	std::string music_name;
	double elapsed_seconds;  //! current offset into the track
	bool music_is_playing;

};

#endif // _AUDIO_H
