/*
 * Stellarium360 astronomy simulation and visualization
 *
 * Copyright (C) 2014_2015 Association Sirius & LSS team 
 * Stellarium360 astronomy simulation and visualization
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Stellarium360 is a free open project of of LSS team
 * See the TRADEMARKS file for free open project usage requirements.
 *
 */
#include "checkkeys.hpp"
 
 void checkConfigIni(string fullpathfile, string _VERSION)
{
	//add all key introducted since 14.4.1
	InitParser user_conf;
	user_conf.load(fullpathfile);
	
	if (user_conf.get_str("main:version") == _VERSION) 
		return; //(nothing to do, config.ini isn't outdated)
	
	//  ****************** main section  ****************** 
	string entry="main";
	user_conf.set_str(entry+":version", _VERSION);
	
	if (!user_conf.find_entry(entry+":flag_optoma"))
		user_conf.set_boolean(entry+":flag_optoma", false);

	if (!user_conf.find_entry(entry+":milkyway_iris"))
		user_conf.set_boolean(entry+":milkyway_iris", false);

	if (!user_conf.find_entry(entry+":flag_masterput"))
		user_conf.set_boolean(entry+":flag_masterput",false);
		
	if (!user_conf.find_entry(entry+":flag_navigation"))
		user_conf.set_boolean(entry+":flag_navigation", false);

	//  ****************** io section  ****************** 
	entry="io";
	if (!user_conf.find_entry(entry+":enable_tcp"))
		user_conf.set_boolean(entry+":enable_tcp", false);
		
	if (!user_conf.find_entry(entry+":enable_mkfifo"))
		user_conf.set_boolean(entry+":enable_mkfifo", false);
		
	if (!user_conf.find_entry(entry+":tcp_port_in"))
		user_conf.set_int(entry+":tcp_port_in", 7805);
		
	if (!user_conf.find_entry(entry+":tcp_buffer_in_size"))
		user_conf.set_int(entry+":tcp_buffer_in_size", 1024);
		
	if (!user_conf.find_entry(entry+":mkfifo_file_in"))
		user_conf.set_str(entry+":mkfifo_file_in", "/tmp/stellarium360_in.fifo");
		
	if (!user_conf.find_entry(entry+":mkfifo_buffer_in_size"))
		user_conf.set_int(entry+":mkfifo_buffer_in_size", 1024);

	//  ****************** navigation section  ****************** 
	entry="navigation";
	if (!user_conf.find_entry(entry+":flag_enable_drag_mouse"))
		user_conf.set_str(entry+":flag_enable_drag_mouse", "true");

	//  ****************** font section  ****************** 
	entry="font";
	if (!user_conf.find_entry(entry+":font_general_name"))
		user_conf.set_str(entry+":font_general_name", "DejaVuSansMono.ttf");

	if (!user_conf.find_entry(entry+":font_general_size"))
		user_conf.set_int(entry+":font_general_size", 12);

	if (!user_conf.find_entry(entry+":font_menu_name"))
		user_conf.set_str(entry+":font_menu_name", "DejaVuSans.ttf");

	if (!user_conf.find_entry(entry+":font_menutui_size"))
		user_conf.set_int(entry+":font_menutui_size", 18);

	if (!user_conf.find_entry(entry+":font_menugui_size"))
		user_conf.set_double(entry+":font_menugui_size", 12.5);

	if (!user_conf.find_entry(entry+":font_planet_name"))
		user_conf.set_str(entry+":font_planet_name", "DejaVuSans.ttf");

	if (!user_conf.find_entry(entry+":font_planet_size"))
		user_conf.set_int(entry+":font_planet_size", 20);

	if (!user_conf.find_entry(entry+":font_constellation_name"))
		user_conf.set_str(entry+":font_constellation_name", "DejaVuSans.ttf");

	if (!user_conf.find_entry(entry+":font_constellation_size"))
		user_conf.set_int(entry+":font_constellation_size", 20);

	//  ****************** stars section  ****************** 
	entry="stars";
	if (!user_conf.find_entry(entry+":illuminate_size"))
		user_conf.set_int(entry+":illuminate_size", 125);

	//  ****************** viewing section  ****************** 
	entry="viewing";
	if (!user_conf.find_entry(entry+":nebula_picto_size"))
		user_conf.set_int(entry+":nebula_picto_size", 6);
		
	//  ****************** rendering section  ******************
	entry="rendering";
	if (!user_conf.find_entry(entry+":antialiasing"))
		user_conf.set_int(entry+":antialiasing", 4);

	//  ****************** gui section  ****************** 
	entry="gui";
	if (!user_conf.find_entry(entry+":flag_show_latlon"))
		user_conf.set_boolean(entry+":flag_show_latlon", false);

	if (!user_conf.find_entry(entry+":flag_mouse_usable_in_script"))
		user_conf.set_boolean(entry+":flag_mouse_usable_in_script", false);

	if (!user_conf.find_entry(entry+":datetime_display_position"))
		user_conf.set_int(entry+":datetime_display_position", 105);

	if (!user_conf.find_entry(entry+":flag_show_planetname"))
		user_conf.set_boolean(entry+":flag_show_planetname", false);

	if (!user_conf.find_entry(entry+":flag_number_print"))
		user_conf.set_int(entry+":flag_number_print", 1);

	//  ****************** color section  ****************** 
	entry="color";
	if (!user_conf.find_entry(entry+":ecliptic_center_color"))
		user_conf.set_boolean(entry+":ecliptic_center_color", "0.8,0.8,0.8");
		
	if (!user_conf.find_entry(entry+":galactic_pole_color"))
		user_conf.set_str(entry+":galactic_pole_color", "0.8,0.8,0.8");

	if (!user_conf.find_entry(entry+":galactic_center_color"))
		user_conf.set_str(entry+":galactic_center_color", "0.8,0.8,0.8");

	if (!user_conf.find_entry(entry+":vernal_points_color"))
		user_conf.set_str(entry+":vernal_points_color", "0.8,0.8,0.8");

	//  ****************** nautical section  ****************** 
	entry="nautical";
	if (!user_conf.find_entry(entry+":flag_nav"))
		user_conf.set_boolean(entry+":flag_nav", false);

	if (!user_conf.find_entry(entry+":flag_show_latlon"))
		user_conf.set_boolean(entry+":flag_show_latlon", false);

	user_conf.save(fullpathfile);
}

/*
 * 	//  ****************** video section  ****************** 
	
	entry="video";
	if (!user_conf.find_entry(entry+":fullscreen"))
		user_conf.set_str(entry+":fullscreen", default_conf.get_str(entry,"fullscreen","false"));
		
	if (!user_conf.find_entry(entry+":screen_w"))
		user_conf.set_str(entry+":screen_w", default_conf.get_str(entry,"screen_w","800"));	
		
	if (!user_conf.find_entry(entry+":screen_h"))
		user_conf.set_str(entry+":screen_h", default_conf.get_str(entry,"screen_h","600"));		
		
	if (!user_conf.find_entry(entry+":bbp_mode"))
		user_conf.set_str(entry+":bbp_mode", default_conf.get_str(entry,"bbp_mode","24"));
		
	if (!user_conf.find_entry(entry+":horizontal_offset"))
		user_conf.set_str(entry+":horizontal_offset", default_conf.get_str(entry,"horizontal_offset","0"));	
		
	if (!user_conf.find_entry(entry+":vertical_offset"))
		user_conf.set_str(entry+":vertical_offset", default_conf.get_str(entry,"vertical_offset","0"));		
		
	if (!user_conf.find_entry(entry+":bbp_mode"))
		user_conf.set_str(entry+":bbp_mode", default_conf.get_str(entry,"bbp_mode","24"));
		
	if (!user_conf.find_entry(entry+":maximum_fps"))
		user_conf.set_str(entry+":maximum_fps", default_conf.get_str(entry,"maximum_fps","60"));	
		
	if (!user_conf.find_entry(entry+":minimum_fps"))
		user_conf.set_str(entry+":minimum_fps", default_conf.get_str(entry,"minimum_fps","30"));		
				
	if (!user_conf.find_entry(entry+":center_horizontal_offset"))
		user_conf.set_str(entry+":center_horizontal_offset", default_conf.get_str(entry,"center_horizontal_offset","0"));	

	//  ****************** projection section  ****************** 

	entry="projection";	
	if (!user_conf.find_entry(entry+":type"))
		user_conf.set_str(entry+":type", default_conf.get_str(entry,"type","fisheye"));
		
	if (!user_conf.find_entry(entry+":viewport"))
		user_conf.set_str(entry+":viewport", default_conf.get_str(entry,"viewport","disk"));	
		
	if (!user_conf.find_entry(entry+":projector_configuration"))
		user_conf.set_str(entry+":projector_configuration", default_conf.get_str(entry,"screen_h","1"));		
		
	if (!user_conf.find_entry(entry+":projector_offset_x"))
		user_conf.set_str(entry+":projector_offset_x", default_conf.get_str(entry,"bbp_mode","0"));
		
	if (!user_conf.find_entry(entry+":projector_offset_y"))
		user_conf.set_str(entry+":projector_offset_y", default_conf.get_str(entry,"horizontal_offset","0"));	
		
	if (!user_conf.find_entry(entry+":horizontal_shear"))
		user_conf.set_str(entry+":horizontal_shear", default_conf.get_str(entry,"vertical_offset","1"));		
		
	if (!user_conf.find_entry(entry+":max_fov"))
		user_conf.set_str(entry+":max_fov", default_conf.get_str(entry,"bbp_mode","360"));




*/
