/*
 * Stellarium360 astronomy simulation and visualization
 *
 * Copyright (C) 2002 Fabien Chereau
 * Copyright (C) 2009 Digitalis Education Solutions, Inc.
 * Copyright (C) 2013 of the LSS team
 * Copyright (C) 2014 of the LSS Team & Association Sirius
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Stellarium360 is a free open project of of LSS team
 * See the TRADEMARKS file for free open project usage requirements.
 *
 */

#ifndef __CARDINALS_H__
#define __CARDINALS_H__

#include <string>
#include <fstream>
//#include "stellarium360.hpp"
#include "s_font.hpp"
#include "projector.hpp"
//#include "navigator.hpp"
//#include "tone_reproductor.hpp"
#include "fader.hpp"
#include "translator.hpp"
//#include "shared_data.hpp"

//! Class which manages the cardinal points displaying
class Cardinals
{
    public:
        Cardinals(float _radius = 1.);
        virtual ~Cardinals();
        void draw(const Projector* prj, double latitude, bool gravityON = false) const;
        void setColor(const Vec3f& c) {
            color = c;
            //~ SettingsState state;
            //~ state.m_state.cardinal_points[0] = c[0];
            //~ state.m_state.cardinal_points[1] = c[1];
            //~ state.m_state.cardinal_points[2] = c[2];
            //~ SharedData::Instance()->Settings(state);
        }
        Vec3f get_color() {
            return color;
        }
        void set_font(float font_size, const string& font_name);
        void translateLabels(Translator& trans);  // for i18n
        void update(int delta_time) {
            fader.update(delta_time);
        }
        void set_fade_duration(float duration) {
            fader.set_duration((int)(duration*1000.f));
        }
        void setFlagShow(bool b) {
            fader = b;
            //~ ReferenceState state;
            //~ state.cardinal_points = b;
            //~ SharedData::Instance()->References( state );
        }
        bool getFlagShow(void) const {
            return fader;
        }
        
        void setInternalNav (bool a) {
			internalNav=a;
		}

    private:
        float radius;
        s_font* font;
        Vec3f color;
        string sNorth, sSouth, sEast, sWest;
        LinearFader fader;
        bool internalNav;
};


#endif // __CARDINALS_H__
