/*
 * Stellarium360 astronomy simulation and visualization
 *
 * Copyright (C) 2014 of the LSS Team & Association Sirius
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Stellarium360 is a free open project of of LSS team
 * See the TRADEMARKS file for free open project usage requirements.
 *
 */

#include <iostream>
#include "illuminate.hpp"
#include "s_texture.hpp"
#include "stellarium360.hpp"
#include "navigator.hpp"
#include "log.hpp"

s_texture * Illuminate::illuminateTex= NULL;


Illuminate::Illuminate()
{
	Name = "";
}

Illuminate::~Illuminate()
{
}


// Read Illuminate data passed in and compute x,y and z;
bool Illuminate::createIlluminate(double ra, double de, double angular_size, string name, double r, double g, double b)
{
	Name = name;
	texColor.set(r,g,b);

	//cout << name << " " << ra << " " << de << " " << angular_size << " " << r << " " << g << " " << b << endl;

	// Calc the RA and DE from the datas - keep base info for drawing (in radians)
	myRA = ra*M_PI/180.;
	myDe = de*M_PI/180.;

	// Calc the Cartesian coord with RA and DE
	sphe_to_rect(myRA,myDe,XYZ);

	float tex_size = sin(angular_size/2/60*M_PI/180);

	// Precomputation of the rotation/translation matrix
	Mat4f mat_precomp = Mat4f::translation(XYZ) *
	                    Mat4f::zrotation(myRA) *
	                    Mat4f::yrotation(-myDe);

	texQuadVertex[0] = mat_precomp * Vec3f(0.,-tex_size,-tex_size); 
	texQuadVertex[1] = mat_precomp * Vec3f(0., tex_size,-tex_size);
	texQuadVertex[2] = mat_precomp * Vec3f(0.,-tex_size, tex_size);
	texQuadVertex[3] = mat_precomp * Vec3f(0., tex_size, tex_size);
	return true;
}


// Read Illuminate data from string and compute x,y and z returns false if can't parse record
bool Illuminate::createIlluminate(const string& record)
{
	string name;
	float ra;
	float de;
	float tex_angular_size;
	double r, g, b;

	std::istringstream istr(record);
	
	if (!(istr >> ra >> de >> tex_angular_size >> name >> r>> g >> b)) return false;

	return createIlluminate(ra, de, tex_angular_size, name, r,g,b);
}

void Illuminate::drawTex(const Projector* prj, const Navigator* nav)
{
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	if (!illuminateTex) return;

	glColor4f(texColor[0], texColor[1], texColor[2], 1.0);
	glBindTexture(GL_TEXTURE_2D, illuminateTex->getID());

	Vec3d v;

	glBegin(GL_TRIANGLE_STRIP);
		glTexCoord2i(1,0);              // Bottom Right
		prj->project_j2000(texQuadVertex[0],v); glVertex3dv(v);
	    glTexCoord2i(0,0);              // Bottom Left
		prj->project_j2000(texQuadVertex[1],v); glVertex3dv(v);
	    glTexCoord2i(1,1);              // Top Right
		prj->project_j2000(texQuadVertex[2],v); glVertex3dv(v);
	    glTexCoord2i(0,1);              // Top Left
		prj->project_j2000(texQuadVertex[3],v); glVertex3dv(v);
	glEnd();
}
