/*
 * Stellarium360 astronomy simulation and visualization
 *
 * Copyright (C) 2015 of the LSS Team & Association Sirius
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Stellarium360 is a free open project of of LSS team
 * See the TRADEMARKS file for free open project usage requirements.
 *
 */

#ifndef _ILLUMINATE_MGR_H_
#define _ILLUMINATE_MGR_H_

#include <vector>
#include "object.hpp"
#include "fader.hpp"
#include "grid.hpp"
#include "illuminate.hpp"

using namespace std;

 /*! \class IlluminateMgr
   * \brief IlluminateMgr handles all illumiante stars for better stars visualisation.
   */
class IlluminateMgr
{
public:
	IlluminateMgr();
	virtual ~IlluminateMgr();

	//! search by name and return an Illuminate object
	Illuminate *searchIlluminate(const string& name);  

	//! Load an individual Illuminate from a script
	bool loadIlluminate(double ra, double de, double angular_size, string name, double r, double g, double b);

	//! remove user added Illuminate and optionally unhide the original of the same name
	string removeIlluminate(const string& name);
	
	//! remove all user added Illuminate
	string removeAllIlluminate();

	//! Draw all the Illuminate
	void draw(Projector *prj, const Navigator *nav);

private:
	vector<Illuminate*> illuminateArray; 		//!< The Illuminate list
	vector<Illuminate*>* illuminateZones;		//!< array of Illuminate vector with the grid id as array rank
	littleGrid illuminateGrid;					//!< Grid for opimisation
};

#endif // _ILLUMINATE_MGR_H_
