/*
 * Stellarium360 astronomy simulation and visualization
 *
 * Copyright (C) 2002 Fabien Chereau
 * Copyright (C) 2009 Digitalis Education Solutions, Inc.
 * Copyright (C) 2013 of the LSS team
 * Copyright (C) 2014 of the LSS Team & Association Sirius
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Stellarium360 is a free open project of of LSS team
 * See the TRADEMARKS file for free open project usage requirements.
 *
 */

#ifndef __MILKYWAY_H__
#define __MILKYWAY_H__

#include <string>
#include <fstream>
#include "stellarium360.hpp"
#include "s_font.hpp"
#include "projector.hpp"
#include "navigator.hpp"
#include "tone_reproductor.hpp"
#include "fader.hpp"
//#include "translator.hpp"


//! Class which manages the displaying of the Milky Way
class MilkyWay
{
    public:
        MilkyWay(float _radius = 1.);
        virtual ~MilkyWay();
        void draw(ToneReproductor * eye, const Projector* prj, const Navigator* nav) const;
        void update(int delta_time) {
            fader.update(delta_time);
        }
        void set_intensity(float _intensity){
			intensityActual = _intensity;
		}

        void set_initial_intensity(float _intensity){
			intensityInitial = _intensity;
		}

        float get_intensity() {
            return intensityActual;
        };
        void set_texture(const string& tex_file, bool blend = false, bool make_default = false);
        void setColor(const Vec3f& c) {
            color=c;
        }
        void setFlagShow(bool b) {
            fader = b;
        }
        bool getFlagShow(void) const {
            return fader;
        }

	void setFaderDuration(float f) {
		fader.set_duration(f);
	}

	void restore_intensity(){
		intensityActual = intensityInitial;
	}

    private:
        float radius;
        s_texture* tex;
        s_texture* default_tex;  //! Default milky way texture can always fall back to
        Vec3f color;
        float intensityActual;
        float intensityInitial;
        LinearFader fader;
};

#endif // __MILKYWAY_H__
