/*
 * Stellarium360 astronomy simulation and visualization
 *
 * Copyright (C) 2002 Fabien Chereau
 * Copyright (C) 2009 Digitalis Education Solutions, Inc.
 * Copyright (C) 2013 of the LSS team
 * Copyright (C) 2014 of the LSS Team & Association Sirius
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Stellarium360 is a free open project of of LSS team
 * See the TRADEMARKS file for free open project usage requirements.
 *
 */

#ifndef __SKYGRID_H__
#define __SKYGRID_H__

#include <string>
#include <fstream>
#include "stellarium360.hpp"
#include "s_font.hpp"
#include "projector.hpp"
#include "navigator.hpp"
//#include "tone_reproductor.hpp"
#include "fader.hpp"
//#include "translator.hpp"
//#include "shared_data.hpp"

//! Class which manages a grid to display in the sky
class SkyGrid
{
    public:
        enum SKY_GRID_TYPE {
            EQUATORIAL,
            ECLIPTIC,
            GALACTIC,
            ALTAZIMUTAL
        };
        // Create and precompute positions of a SkyGrid
        SkyGrid(SKY_GRID_TYPE grid_type = EQUATORIAL, unsigned int _nb_meridian = 24, unsigned int _nb_parallel = 17,
                double _radius = 1., unsigned int _nb_alt_segment = 18, unsigned int _nb_azi_segment = 50);
        virtual ~SkyGrid();
        void draw(const Projector* prj) const;
        void set_font(float font_size, const string& font_name);
        void setColor(const Vec3f& c) {
            color = c;
            //~ SettingsState state;
            //~ if( gtype == EQUATORIAL ) {
                //~ state.m_state.equator_grid[0] = c[0];
                //~ state.m_state.equator_grid[1] = c[1];
                //~ state.m_state.equator_grid[2] = c[2];
            //~ }
            //~ if( gtype == ECLIPTIC ) {
                //~ state.m_state.ecliptic_grid[0] = c[0];
                //~ state.m_state.ecliptic_grid[1] = c[1];
                //~ state.m_state.ecliptic_grid[2] = c[2];
            //~ }
//~ 
            //~ if( gtype == GALACTIC ) {
                //~ state.m_state.galactic_grid[0] = c[0];
                //~ state.m_state.galactic_grid[1] = c[1];
                //~ state.m_state.galactic_grid[2] = c[2];
            //~ }
            //~ else if( gtype == ALTAZIMUTAL ) {
                //~ state.m_state.azimuth_grid[0] = c[0];
                //~ state.m_state.azimuth_grid[1] = c[1];
                //~ state.m_state.azimuth_grid[2] = c[2];
            //~ }
            //~ SharedData::Instance()->Settings(state);
        }
        const Vec3f& getColor() {
            return color;
        }
        void update(int delta_time) {
            fader.update(delta_time);
        }
        void set_fade_duration(float duration) {
            fader.set_duration((int)(duration*1000.f));
        }
        void setFlagshow(bool b) {
            fader = b;
            //~ ReferenceState state;
            //~ if( gtype == ALTAZIMUTAL )
                //~ state.azimuthal_grid = b;
            //~ else if( gtype == EQUATORIAL )
                //~ state.equatorial_grid = b;
            //~ else if( gtype == ECLIPTIC )
                //~ state.ecliptic_grid = b;
            //~ else if( gtype == GALACTIC )
                //~ state.ecliptic_grid = b;
            //~ SharedData::Instance()->References( state );
        }
        bool getFlagshow(void) const {
            return fader;
        }
        void set_top_transparancy(bool b) {
            transparent_top= b;
        }
        
        void setInternalNav (bool a) {
			internalNav=a;
		}
    private:

        unsigned int nb_meridian;
        unsigned int nb_parallel;
        double radius;
        unsigned int nb_alt_segment;
        unsigned int nb_azi_segment;
        bool transparent_top;
        Vec3f color;
        Vec3f** alt_points;
        Vec3f** azi_points;
        bool (Projector::*proj_func)(const Vec3d&, Vec3d&) const;
        s_font* font;
        bool internalNav;
        SKY_GRID_TYPE gtype;
        LinearFader fader;
};


#endif // __SKYGRID_H__
