/*
 * Stellarium360 astronomy simulation and visualization
 *
 * Copyright (C) 2002 Fabien Chereau
 * Copyright (C) 2009 Digitalis Education Solutions, Inc.
 * Copyright (C) 2013 of the LSS team
 * Copyright (C) 2014 of the LSS Team & Association Sirius
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Stellarium360 is a free open project of the LSS team
 * See the TRADEMARKS file for free open project usage requirements.
 *
 */

#include "entity.hpp"
#include "skyperson.hpp"
#include "s_texture.hpp"
#include "utility.hpp"
#include <string>
#include "log.hpp"

using namespace std;

// -------------------- SKYLINE_PERSONAL  ---------------------------------------------

SkyPerson::SkyPerson(PERSON_TYPE ptype)
{
	string homeDir = getenv("HOME");
	switch(ptype)
	{
	case AL: proj_func = &Projector::project_local;
			break;
	case EQ: proj_func = &Projector::project_earth_equ;
			break;
	default: proj_func = &Projector::project_local;
			break;
	}
}

SkyPerson::~SkyPerson()
{
}

void SkyPerson::clear() const
{
	nblines=0;
}

void SkyPerson::loadData(string filename) const
{
	clear();
	string homeDir = getenv("HOME");
	string action = homeDir+"/."+APP_LOWER_NAME +"/"+ filename;
	
	int err=0;
	tempFile = fopen(action.c_str(),"r");	
    if (tempFile) {
		
		//read the number of line for the file
		if (fscanf(tempFile,"%i",&nblines) == 0) {
			Log.write("SkyPerson mismatch format", cLog::eLOG_TYPE_WARNING);	
			return;
		}
		if (nblines>=NB_POINTS) nblines=NB_POINTS;

		//read each points
		for (/*unsigned*/ int i=0; i<nblines; i++) {
			err=err+ fscanf(tempFile,"%f %f",&alpha,&delta); //todo move fscanf and use fstream
			sphe_to_rect(alpha,delta,punts[i]);
		}
		if (err!=0)	
			Log.write("SkyPerson: error parsing file "+ action, cLog::eLOG_TYPE_WARNING);
		fclose(tempFile);
	}
	else
	Log.write("SkyPerson Unable to open " + action, cLog::eLOG_TYPE_WARNING);
}

void SkyPerson::draw(const Projector *prj,const Navigator *nav) const
{
	if (!fader.getInterstate()) return;

    string homeDir = getenv("HOME");
	glColor4f(color[0], color[1], color[2], fader.getInterstate());
	glDisable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); // Normal transparency mode

	prj->set_orthographic_projection();	// set 2D coordinate

	for (int i=0; i < nblines; i=i+2) {

		if ((prj->*proj_func)(punts[i], pt1) &&	(prj->*proj_func)(punts[i+1], pt2) ) {
			glBegin (GL_LINES);
			glVertex2f(pt1[0],pt1[1]);
			glVertex2f(pt2[0],pt2[1]);
			glEnd();
		}
	}

	prj->reset_perspective_projection();
}
