/*
 * Stellarium360 astronomy simulation and visualization
 *
 * Copyright (C) 2002 Fabien Chereau
 * Copyright (C) 2009 Digitalis Education Solutions, Inc.
 * Copyright (C) 2013 of the LSS team
 * Copyright (C) 2014 of the LSS Team & Association Sirius
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Stellarium360 is a free open project of the LSS team
 * See the TRADEMARKS file for free open project usage requirements.
 *
 */

#include "suntrace.hpp"
#include "utility.hpp"
#include <string>

using namespace std;


SunTrace::SunTrace(SolarSystem * _system)
{
	for(int i= 0; i<NB_MAX_LIST; i++)
	{
		sundata[i].size=0;
		sundata[i].old_punt[0]=0.0;
		sundata[i].old_punt[1]=0.0;
		sundata[i].color[0]=1.;
		sundata[i].color[1]=0.;
		sundata[i].color[2]=0.;
		sundata[i].hide=false;
	}
	sundata[0].color[0]=1.;
	sundata[0].color[1]=0.;
	sundata[0].color[2]=0.;
	
	sundata[1].color[0]=0.2;
	sundata[1].color[1]=0.2;
	sundata[1].color[2]=0.7;
	
	sundata[2].color[0]=0.5;
	sundata[2].color[1]=0.9;
	sundata[2].color[2]=0.4;
	
	sundata[3].color[0]=0.3;
	sundata[3].color[1]=0.6;
	sundata[3].color[2]=0.1;

	sundata[4].color[0]=0.1;
	sundata[4].color[1]=0.8;
	sundata[4].color[2]=0.5;
	
	sundata[5].color[0]=0.;
	sundata[5].color[1]=1.;
	sundata[5].color[2]=0.2;

	sundata[6].color[0]=0.2;
	sundata[6].color[1]=1.;
	sundata[6].color[2]=0.4;
	
	sundata[7].color[0]=0.4;
	sundata[7].color[1]=1.;
	sundata[7].color[2]=0.6;

	is_tracing=true;
	currentUsedList=0;
	ssystem= _system;
}

SunTrace::~SunTrace()
{
}

void SunTrace::hide(int numberList)
{
	if ( numberList > (NB_MAX_LIST-1)) return;
	if (numberList==-1){
		//-1 mean all the lists
		for(int i=0;i<NB_MAX_LIST; i++)
			sundata[i].hide= !sundata[i].hide;
		}
	else
		sundata[numberList].hide= !sundata[numberList].hide;
}

void SunTrace::draw(const Projector *prj,const Navigator *nav) const
{
	if (!fader.getInterstate()) return;
    glDisable(GL_TEXTURE_2D);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); // Normal transparency mode

	prj->set_orthographic_projection();	// set 2D coordinate
	for(int l=0; l<currentUsedList+1; l++)
	{
		if (sundata[l].size>2 && !sundata[l].hide) {
//		glColor4f(sundata[l].color[0], sundata[l].color[1], sundata[l].color[2], fader.getInterstate());
		glColor3f(sundata[l].color[0],sundata[l].color[1],sundata[l].color[2]);
		for (int i=0; i < sundata[l].size; i=i+1)
			{
                if ((prj->project_local(sundata[l].punts[i], pt1)) && (prj->project_local(sundata[l].punts[i+1], pt2)) ) {
				glBegin (GL_LINES);
				glVertex2f(pt1[0],pt1[1]);
				glVertex2f(pt2[0],pt2[1]);
				glEnd();
			}
		}
	}
	}
	prj->reset_perspective_projection();
}

void SunTrace::addData(const Navigator *nav)
{
	if (!fader.getInterstate()) return;
	if(!is_tracing) return;
	
	double alt, az;
	ssystem->getSunTraceAltAz(nav, &alt, &az);

	if (sundata[currentUsedList].size==0)
		{
			sundata[currentUsedList].old_punt[0]=alt;
			sundata[currentUsedList].old_punt[1]=az;
			sphe_to_rect(-(az+M_PI),alt,sundata[currentUsedList].punts[0]);
			sundata[currentUsedList].size=1;
		}
	else
	{
		if ( (abs(alt-sundata[currentUsedList].old_punt[0])< 0.001) && (abs(az-sundata[currentUsedList].old_punt[1])< 0.001) ) return;
		if (sundata[currentUsedList].size==(MAX_POINTS-1)) return;
		sundata[currentUsedList].old_punt[0]=alt;
		sundata[currentUsedList].old_punt[1]=az;
		sundata[currentUsedList].size+=1;
		sphe_to_rect(-(az+M_PI),alt,sundata[currentUsedList].punts[sundata[currentUsedList].size]);
//		printf("Point %i sur liste %i \n",sundata[currentUsedList].size, currentUsedList);
	}
}
