/*
 * Stellarium360 astronomy simulation and visualization
 *
 * Copyright (C) 2015 of the LSS Team & Association Sirius
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Stellarium360 is a free open project of of LSS team
 * See the TRADEMARKS file for free open project usage requirements.
 *
 */

#ifndef __SUNTRACE_H__
#define __SUNTRACE_H__

#include <string>
#include <fstream>
#include "projector.hpp"
#include "navigator.hpp"
#include "fader.hpp"
#include "solarsystem.hpp"

#define NB_MAX_LIST 7
#define MAX_POINTS 16384


//! Class which manages a line to display the Sun position around the sky
class SunTrace
{
    public:
		struct SunList{
			Vec3f color;
			int size;
			Vec3f punts[MAX_POINTS];
			Vec2f old_punt;
			bool hide;
		};
    
        SunTrace(SolarSystem * _ssystem);
        virtual ~SunTrace();
        void draw(const Projector *prj,const Navigator *nav) const;
        
        void setColor(const Vec3f& c, int numberlist) {
            sundata[numberlist].color = c;
        }

        const Vec3f& getColor( int numberlist) {
            return sundata[numberlist].color;
        }
        
        void hide(int numberList);
        
        void update(int delta_time) {
            fader.update(delta_time);
        }

        void set_fade_duration(float duration) {
            fader.set_duration((int)(duration*1000.f));
        }

        void setFlagShow(bool b) {
            fader = b;
            if (b==false)
				clear();
        }

        bool getFlagShow(void) const {
            return fader;
        }

		void addData(const Navigator *nav);

        void setPen(bool b) {
            is_tracing = b;
            if (b==false) 
				currentUsedList+=1;
			if (currentUsedList==NB_MAX_LIST)
				currentUsedList=NB_MAX_LIST-1;
        }

        bool getPen(void) const {
            return is_tracing;
        }


		void clear() {
			currentUsedList=0;
				for(int i= 0; i<NB_MAX_LIST; i++)
					sundata[i].size=0;
		}
		//void loadSunData(string filename, int numberlist); a venir
		
		//void saveSunData(string filename, int whatlist); a venir
		
		private:
			SunList sundata[NB_MAX_LIST];
			bool is_tracing;
			int currentUsedList;
	        LinearFader fader;
			mutable Vec3d pt1;
	        mutable Vec3d pt2;
			SolarSystem *ssystem;
};


#endif // __SUNTRACE_H__


