/*
 * Stellarium360 astronomy simulation and visualization
 *
 * Copyright (C) 2014 of the LSS Team & Association Sirius
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Stellarium360 is a free open project of of LSS team
 * See the TRADEMARKS file for free open project usage requirements.
 *
 */

// Class which handles  Text for User script
#include <string>
#include <fstream>
#include <iostream>

#include "log.hpp"
#include "text_mgr.hpp"

using namespace std;

Text::Text(string _name, string _text, int _altitude, int _azimuth , float _r, float _g, float _b)
{
		name= _name;
		text= _text;
		altitude= _altitude;
		azimuth= _azimuth;
		r=_r;
		g=_g;
		b=_b;
}

Text::~Text()
{
}

void Text::setDisplay(bool b) {
	fader=b;
}

bool Text::getDisplay(void) const {
	return fader;
}

std::string Text::getName() {
	return name;
}

void Text::draw(Projector* prj, s_font *textFont)
{
	glEnable(GL_TEXTURE_2D);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_BLEND);
	
	if ( !fader.getInterstate() ) return;
	glColor4f(r,g,b,fader.getInterstate());
	textFont->print_gravity(prj, altitude, azimuth, text);
}

void Text::update(int delta_time)
{
	fader.update(delta_time);
}


void Text::text_update(std::string _text)
{
	text=_text;
}

Text_Mgr::Text_Mgr()
{
	textFont=NULL;
}

Text_Mgr::~Text_Mgr()
{
	vector<Text *>::iterator iter;
		for (iter = textUsr.begin(); iter != textUsr.end(); iter++)
			delete(*iter);

	if (textFont) delete textFont;
	textFont = NULL;
}

void Text_Mgr::update(int delta_time)
{
	vector < Text * >::const_iterator iter;
	for (iter = textUsr.begin(); iter != textUsr.end(); ++iter) {
		(*iter)->update(delta_time);
	}
}

bool Text_Mgr::add(std::string name, std::string text, int altitude, int azimuth, float r, float g, float b)
{
	Text *token =new Text(name, text, altitude, azimuth, r,g,b);
	if (token != NULL)
	{
		textUsr.push_back(token);
		return true;
	}
	else
		Log.write("TEXT: Bad text syntax", cLog::eLOG_TYPE_ERROR);
		return false;
}

void Text_Mgr::clear()
{
	vector < Text * >::iterator iter;
	for (iter = textUsr.begin(); iter != textUsr.end(); ++iter) {
		delete(*iter);
	}
	textUsr.clear();
}

bool Text_Mgr::del(std::string name)
{
	vector < Text * >::iterator iter;
	for (iter = textUsr.begin(); iter != textUsr.end(); ++iter) {
		if ((*iter)->getName() == name)
		{
			delete(*iter);
			textUsr.erase(iter);
			return true;
		}
	}
	return false;
}

void Text_Mgr::name_update(string name, string text)
{
	vector < Text * >::iterator iter;
	for (iter = textUsr.begin(); iter != textUsr.end(); ++iter) {
		if ((*iter)->getName() == name)
		{
			(*iter)->text_update(text);
			return;
		}
	}
}

void Text_Mgr::text_display(string name , bool displ)
{
	vector < Text * >::iterator iter;
	for (iter = textUsr.begin(); iter != textUsr.end(); ++iter) {
		if ((*iter)->getName() == name)
		{
			(*iter)->setDisplay(displ);
			return;
		}
	}
}

void Text_Mgr::setFadingDuration(int t)
{
	vector < Text * >::iterator iter;
	for (iter = textUsr.begin(); iter != textUsr.end(); ++iter) {
			(*iter)->setFadingDuration(t);
	}
}

void Text_Mgr::setFont(float font_size, const string& font_name)
{
	if (textFont) delete textFont;
	textFont = new s_font(font_size, font_name);
	if (!textFont) {
		Log.write("TEXT: can't create text usr font", cLog::eLOG_TYPE_ERROR);
		exit(-1);
	}
}


void Text_Mgr::draw(Projector* prj)
{
	vector < Text * >::iterator iter;
	for (iter = textUsr.begin(); iter != textUsr.end(); ++iter) {
		(*iter)->draw(prj,textFont);
	}
}
