/*
 * Stellarium360 astronomy simulation and visualization
 *
 * Copyright (C) 2014 of the LSS Team & Association Sirius
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Stellarium360 is a free open project of of LSS team
 * See the TRADEMARKS file for free open project usage requirements.
 *
 */

#ifndef _TEXT_MGR_H
#define _TEXT_MGR_H

#include "s_font.hpp"
#include "fader.hpp"
#include "projector.hpp"
#include <vector>

//cette classe s'occupe d'un seul text à afficher 
class Text
{
	public:
		Text(string _name, string _text, int _altitude, int _azimuth, float _r, float _g, float _b);
		~Text();
		void setDisplay(bool b);
		bool getDisplay(void) const;
		std::string getName();
		void text_update(std::string _text);
		void draw(Projector* prj, s_font *textFont);
		void update(int delta_time);

		void setFadingDuration(int a) {
			fader.set_duration(a);
			cout << name<< " "<< a << endl;
		};
	
	private:
		std::string name;
		std::string text;
		float r;
		float g;
		float b;
		int altitude;
		int azimuth;
		LinearFader fader;
};


//cette classe s'occupe de gérer toutes les entitées de text à traiter.
class Text_Mgr
{
	public:
		Text_Mgr();
		~Text_Mgr();
		void update(int delta_time);
		void draw(Projector* prj);
		bool add(std::string name, std::string text, int altitude, int azimuth, float r, float g, float b);
		bool del(std::string name);
		void clear();
		void name_update(std::string name, std::string text);
		void text_display(string name , bool displ);
		void setFadingDuration( int t);
		void setFont(float font_size, const string& font_name);
	private:
		vector<Text *> textUsr;
		s_font *textFont;
};

#endif
