/*
 * Stellarium360 astronomy simulation and visualization
 *
 * Copyright (C) 2015 of the Association Sirius
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Stellarium360 is a free open project of the LSS team
 * See the TRADEMARKS file for free open project usage requirements.
 *
 */

#include <fstream>
#include <iostream>
#include <sstream>
#include <string>
#include "log.hpp"
#include "io.hpp"

#include "stellarium360.hpp"
#include "utility.hpp"

//for pipe
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>




//mkfifo -----------------------


Mkfifo::Mkfifo()
{
//	is_shared_data=false;
	is_active_in=false;
	//is_active_out=false;
	lock_in=NULL;
	//lock_out=NULL;
}

Mkfifo::~Mkfifo()
{
	is_active_in=false;
	//is_active_out=false;
}

void Mkfifo::init_in(string _filename, int _buffer_in_size)
{
	filename_in=_filename;
	buffer_in_size= _buffer_in_size;
	is_active_in= true;
	lock_in=SDL_CreateMutex();
	threadMkfifoRead = SDL_CreateThread(in_thread_wrapper,"ThreadRead", this);
}

//~ void Mkfifo::init_out(string _filename, int _buffer_out_size)
//~ {
	//~ filename_out=_filename;
	//~ buffer_out_size= _buffer_out_size;
	//~ is_active_out= true;
	//~ lock_out=SDL_CreateMutex();
	//~ threadMkfifoWrite = SDL_CreateThread(out_thread_wrapper,"ThreadWrite", this);
//~ }

int Mkfifo::in_thread_wrapper(void *Data)
{
   return ((Mkfifo *)Data)->in_thread(); 
}

//~ int Mkfifo::out_thread_wrapper(void *Data)
//~ {
   //~ return ((Mkfifo *)Data)->out_thread(); 
//~ }

int Mkfifo::in_thread(void )
{
	Log.write("Thread in MKFIFO, buffer_in_size is "+Utility::intToString(buffer_in_size), cLog::eLOG_TYPE_INFO);
	int fdtr;
	char *in;
	in = (char*) calloc (buffer_in_size,sizeof(char)); 
	if (in ==NULL) return 1;
	
	//~ string homeFIFO = getenv("HOME")+string("/essai.fifo");
	//~ cout << "Pipe named  " << filename_in << endl;
	Log.write("Pipe in named  " + filename_in, cLog::eLOG_TYPE_INFO); 
	unlink(filename_in.c_str());
	if (mkfifo((filename_in.c_str()), S_IRWXU| S_IWGRP | S_IWOTH ) == -1) //TODO why result has no g+o=w mode ?
	{
		Log.write("Error creating MkFifo pipe in thread in_thread "+Utility::intToString(errno), cLog::eLOG_TYPE_ERROR);
		return 2;
	}
	else
	{
		Log.write("Creating Mkfifo in pipe successfull", cLog::eLOG_TYPE_INFO); 
		char mode[] = "0777";
		int i = strtol(mode, 0, 8);
		chmod(filename_in.c_str(),i);
	}
    if((fdtr = open (filename_in.c_str(), O_RDONLY )) == -1) 
	{
		Log.write("Unable to open named mkfifo in pipe, error code is "+Utility::intToString(errno), cLog::eLOG_TYPE_ERROR);
		return 2;
	}

    while (is_active_in)
    {
	memset(in,'\0',buffer_in_size);

	if (read(fdtr, in, buffer_in_size) == -1)
    {
		Log.write("Mkfifo read() error "+Utility::intToString(errno), cLog::eLOG_TYPE_ERROR);
		//~ perror("read() error");
		//~ printf ("Code de l'erreur : %d\n", errno);
	}

	if (in[0] !='\0')
		{
			//todo: 2 buffer in 1.
			in[buffer_in_size-1]='\0';
			//printf("moi thread j'ai quelque chose %s", in);
			SDL_LockMutex( lock_in );
			//printf("\ntaille de queue : %d\n", from_outside.size());       
			from_outside.push(in);
			//printf("\n taille de queue : %d\n", from_outside.size());       
			SDL_UnlockMutex( lock_in );
		}
	else // we don't close the pipe until program end but there is no writer.
		sleep(1);
	}
	Log.write("Closing Mkfifo pipe in in_thread", cLog::eLOG_TYPE_INFO); 	
	//~ printf("Closing Mkfifo pipe in in_thread\n");
    close(fdtr);
    free(in); 
	//~ printf("Deleting pipe link\n");
	unlink(filename_in.c_str()); 
    return 0;
}

//~ void Mkfifo::get_for_outside(string string_to_write)
//~ {
	//~ SDL_LockMutex( lock_out );
	//~ for_outside.push(string_to_write);
	//~ SDL_UnlockMutex( lock_out );
//~ }

//~ int Mkfifo::out_thread(void )
//~ {
	//~ Log.write("Thread out MKFIFO, buffer_in_size is "+Utility::intToString(buffer_out_size), cLog::eLOG_TYPE_INFO);
	//~ printf("thread out lancé\n");
	//~ int fdtr;
	//~ char *out;
	//~ bool should_i_wait=true;
	//~ out = (char*) calloc (buffer_out_size,sizeof(char)); 
	//~ if (out ==NULL) return 1;
	//~ 
	//~ string homeFIFO = getenv("HOME")+string("/essai.fifo");
	//~ cout << "Pipe named  " << filename_in << endl;
	//~ Log.write("Pipe out named  " + filename_out, cLog::eLOG_TYPE_INFO); 
	//~ unlink(filename_out.c_str());
	//~ if (mkfifo((filename_out.c_str()), S_IRWXU| S_IWGRP | S_IWOTH ) == -1) //TODO why result has no g+o=w mode ?
	//~ {
		//~ Log.write("Error creating MkFifo pipe out thread out_thread "+Utility::intToString(errno), cLog::eLOG_TYPE_ERROR);
		//~ return 2;
	//~ }
	//~ else
	//~ {
		//~ Log.write("Creating Mkfifo out pipe successfull", cLog::eLOG_TYPE_INFO); 
		//~ char mode[] = "0777";
		//~ int i = strtol(mode, 0, 8);
		//~ chmod(filename_out.c_str(),i);
	//~ }
	//~ 
    //~ if((fdtr = open(filename_out.c_str(), O_WRONLY )) == -1) 
	//~ {
		//~ Log.write("Unable to open named mkfifo out pipe, error code is "+Utility::intToString(errno), cLog::eLOG_TYPE_ERROR);
		//~ return 2;
	//~ }
	//~ 
	//~ printf("lancement mkfifo out reussi\n");
    //~ while (is_active_out)
    //~ {
		//~ should_i_wait=true;
		//~ while (for_outside.size() != 0)
		//~ {
			//~ //printf("taille de la pile dans le thread out est de %3i\t%s\n", for_outside.size(),for_outside.front().c_str());
			//~ memset(out,'\0',buffer_out_size);
			//~ SDL_LockMutex( lock_out);
			//~ strcpy(out,for_outside.front().c_str());
			//~ for_outside.pop();
			//~ SDL_UnlockMutex( lock_out );
			//~ write(fdtr, out, buffer_out_size);
			//~ should_i_wait=false;
		//~ }
		//~ if (should_i_wait)
			//~ SDL_Delay(150); //if nothing to write in the pipe
	//~ }
	//~ Log.write("Closing Mkfifo pipe out out_thread", cLog::eLOG_TYPE_INFO); 	
	//~ printf("Closing Mkfifo pipe in in_thread\n");
    //~ close(fdtr);
    //~ free(out); 
	//~ printf("Deleting pipe link\n");
	//~ unlink(filename_out.c_str()); 
    //~ return 0;
//~ }


bool Mkfifo::update_in(string &output)
{
	if ( (is_active_in ==true))
	{
		if (! from_outside.empty() )
			{
			SDL_LockMutex(lock_in);
			cout << "Mkfifo : I get : " << from_outside.front() << endl;
			output= from_outside.front();//commander->execute_command( mkfifo_in_data.from_outside );
			from_outside.pop();
			SDL_UnlockMutex(lock_in);
			return true;
			}
		else
			return false;
	}
	else 
		return false;
}



//tcp - ip



Tcp::Tcp()
{
	listenfd = 0;
	connfd = 0;
	buffer_in_size=0;
	is_active_in=false;
}

Tcp::~Tcp()
{
	is_active_in=false;
}

void Tcp::init(int tcp_port, int _buffer_in_size)
{
	//~ printf("Tcp Initialisation\n");
	Log.write("Tcp Initialisation", cLog::eLOG_TYPE_INFO); 
	buffer_in_size=_buffer_in_size;
    listenfd = socket(AF_INET, SOCK_STREAM, 0);
    memset(&serv_addr, '0', sizeof(serv_addr));
    
    serv_addr.sin_family = AF_INET;
    serv_addr.sin_addr.s_addr = htonl(INADDR_ANY);
    serv_addr.sin_port = htons(tcp_port);

    bind(listenfd, (struct sockaddr*)&serv_addr, sizeof(serv_addr)); 

    listen(listenfd, 2);
	lock_in=SDL_CreateMutex();
	is_active_in= true;
	threadTcpRead = SDL_CreateThread(in_thread_wrapper,"ThreadRead", this);
}

int Tcp::in_thread_wrapper(void *Data)
{
   return ((Tcp *)Data)->in_thread(); 
}


int Tcp::in_thread(void )
{
	//~ printf("In thread TCP, buffer_in_size is %i\n",buffer_in_size);
	
	char* recvBuff;
	recvBuff = (char*) calloc (buffer_in_size,sizeof(char)); 
	if (recvBuff ==NULL) return 1;
	
	bool is_alive;
	Log.write("Thread TCP wait for connexion", cLog::eLOG_TYPE_INFO); 	
	//~ printf("Thread TCP en fonction, j'attends les connexions\n");
    while (is_active_in)
    {
		Log.write("Thread TCP connexion opened", cLog::eLOG_TYPE_INFO); 
		//~ printf("Thread TCP ouverture de la connection\n");
		connfd = accept(listenfd, (struct sockaddr*)NULL, NULL); 
		is_alive=true;

		while(is_alive)
		{
			memset(recvBuff,'\0', buffer_in_size);
			
			if (recv(connfd, recvBuff, buffer_in_size,0) == -1)
				Log.write("TCP read() error "+Utility::intToString(errno), cLog::eLOG_TYPE_ERROR);
			
			//~ printf("activité sur les thread TCP\n");
			//~ printf("Thread TCP recu: %s\n",recvBuff); 
				Log.write("TCP read() "+ string(recvBuff), cLog::eLOG_TYPE_DEBUG);
			
			if (recvBuff[0] !='\0')
				{
					//todo paquets multiples
					recvBuff[buffer_in_size-1]='\0';
					//printf("Thread Tcp j'ai quelque chose %s\n", recvBuff);
					SDL_LockMutex( lock_in );
					//~ printf("\ntaille de queue : %d\n", from_outside.size());       
					from_outside.push(recvBuff);
					//~ printf("\n taille de queue : %d\n", from_outside.size());       
					SDL_UnlockMutex( lock_in );
					
					SDL_Delay(100);
					//sleep(1);
				}
			else
				is_alive=false; //printf("message vide\n");
		}
		//~ printf("Closed TCP connexion");
		Log.write("Closed TCP connexion", cLog::eLOG_TYPE_INFO); 
		close(connfd);
		//sleep(1);
	}
    free(recvBuff);
	Log.write("Closed TCP socket", cLog::eLOG_TYPE_INFO); 
    return 0;
}


bool Tcp::update(string &output)
{
	if ( (is_active_in ==true))
	{
		if (! from_outside.empty() )
			{
			SDL_LockMutex(lock_in);
			cout << "Tcp : I get : " << from_outside.front() << endl;
			output= from_outside.front();//commander->execute_command( mkfifo_in_data.from_outside );
			from_outside.pop();
			SDL_UnlockMutex(lock_in);
			return true;
			}
		else
			return false;
	}
	else 
		return false;
}



//~ #if LINUX
//~ int App::mkfifo_in_thread_wrapper(void *Data)
//~ {
   //App* self = static_cast<App*>(Data);
   //return self->ThreadFunc();
   //~ return ((App *)Data)->mkfifo_in_thread(); 
//~ }
//~ 
//~ 
//~ int App::mkfifo_in_thread(void )
//~ {
//~ 
	//~ int fdtr;
	//~ char in[4096];
	//~ 
	//~ string homeFIFO = getenv("HOME")+string("/essai.fifo");
	//~ cout << "Tube " << homeFIFO << endl;
	//~ unlink(homeFIFO.c_str());
	//~ if (mkfifo((homeFIFO.c_str()), S_IRWXU| S_IWGRP | S_IWOTH ) == -1) //TODO why result has no g+o=w mode ?
	//~ {
		//~ fprintf(stderr, "Erreur de création du tube\n");
		//~ printf ("Code de l'erreur : %d\n", errno);
		//~ return 1;
	//~ }
	//~ else
	//~ {
		//~ printf("Création du tube réussie\n"); 
		//~ char mode[] = "0777";
		//~ int i = strtol(mode, 0, 8);
		//~ chmod(homeFIFO.c_str(),i);
	//~ }
    //~ if((fdtr = open (homeFIFO.c_str(), O_RDONLY )) == -1) 
	//~ {
		//~ fprintf(stderr, "Impossible d'ouvrir la sortie du tube nommé.\n");
		//~ printf ("Code de l'erreur : %d\n", errno);
		//~ return 2;
	//~ }
//~ 
    //~ while (mkfifo_in_data.active)
    //~ {	
	//~ memset(in,'\0',4096);
//~ 
	//~ if (read(fdtr, in, sizeof(in)) == -1)
    //~ {
		//~ perror("read() error");
        //~ printf ("Code de l'erreur : %d\n", errno);
	//~ }
//~ 
	//~ if (read(fdtr, in, sizeof(in)) == 0)
    //~ {
		//~ perror("read() error");
        //~ printf ("Code de l'erreur : %d\n", errno);
	//~ }
//~ 
//~ 
	//~ if (in[0] !='\0')
		//~ {
			//~ SDL_LockMutex( mkfifo_in_data.lock_in );       
			//~ mkfifo_in_data.from_outside.assign(in);
			//~ mkfifo_in_data.is_shared_data=true;
			//~ SDL_UnlockMutex( mkfifo_in_data.lock_in );
		//~ }
	//~ else
		//~ sleep(1);
//~ 
	//~ }
	//~ printf("fermeture du tube\n");
    //~ close(fdtr); 
	//~ printf("Suppression physique du tube\n");
	//~ unlink(homeFIFO.c_str()); 
    //~ return 0;
//~ }
//~ #endif



/*
StellState::StellState()
{
	scriptState= UNK;
}

StellState::~StellState()
{
}

void StellState::setScriptState(ScriptState _value)
{
	scriptState= _value;
	//~ printf("J'insère quelque chose dans la pile de StellState\n");
	switch (_value)
	{
		case PLAYING: for_output.push("script:PLAYING"); break;
		case STOPPED: for_output.push("script:STOPPED"); break;
		case PAUSED: for_output.push("script:PAUSED"); break;
		case FF: for_output.push("script:FF"); break;
		default: for_output.push("script:UNKNOWN");
	}
	//~ printf("la pile a pour taille %i\n", for_outside.size());
}


bool StellState::updateState(string &string_to_extract)
{
	//~ printf("Dans stellState::updateState taille de la pile est de : %i\n", for_outside.size());
	if (!for_output.empty())
	{
		 //~ printf("Dans Mkfifo stellState::updateState %3i %s\n",for_output.size(), for_output.front().c_str());
	     string_to_extract = for_output.front().c_str();
	     for_output.pop();
	     //~ printf("la pile a alors la taille %i\n", for_outside.size());
	     return true;
    }
    else
		return false;
}
*/
