/*
 * Stellarium360 astronomy simulation and visualization
 *
 * Copyright (C) 2015 of Association Sirius
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Stellarium360 is a free open project of the LSS team
 * See the TRADEMARKS file for free open project usage requirements.
 *
 */

#ifndef IO_H
#define IO_H

#include "sdl_facade.hpp"
#include "SDL2/SDL_thread.h"
#include <queue>
#include "stellarium360.hpp"
#include <cstring>

//tcp
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>


//for pipe
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

using namespace std;

class Mkfifo
{
	public:
		bool is_active_in; //, is_active_out;
		void init_in(string _filename, int _buffer_in_size);
		//void init_out(string _filename, int _buffer_out_size);
		Mkfifo();
		~Mkfifo();
		bool update_in(string &output);
		//void get_for_outside(string string_to_write);
	private:
		int buffer_in_size; /*,buffer_out_size;*/
		queue<string> from_outside;
		//queue<string> for_outside;
		string filename_in; /*, filename_out;*/
		SDL_mutex* lock_in;
		//SDL_mutex* lock_out;
		SDL_Thread* threadMkfifoRead; //create thread for mkfifo
		//SDL_Thread* threadMkfifoWrite; //create thread for mkfifo
		int in_thread(void); //function thread qui gere la lecture des données de l'extérieur
		static int in_thread_wrapper(void *Data); //function what call threadMkfifoRead
		//int out_thread(void); //function thread qui gere la lecture des données de l'intérieur
		//static int out_thread_wrapper(void *Data); //function what call threadMkfifoWrite

};

class Tcp
{
	public:
		bool is_active_in;
		void init(int net_port, int _buffer_in_size);
		Tcp();
		~Tcp();
		bool update(string &output);
	private:
		int buffer_in_size;
		queue<string> from_outside;
		int listenfd, connfd;
		struct sockaddr_in serv_addr; 
		SDL_mutex* lock_in;
		SDL_Thread* threadTcpRead; //create thread for tcp/ip
		int in_thread(void); //function thread qui gere la lecture des données de l'extérieur
		static int in_thread_wrapper(void *Data); //function what call threadTcpIpRead
};


//for output
/*
enum ScriptState { UNK, PLAYING, STOPPED, PAUSED, FF };

class StellState
{
	public:
		StellState();
		~StellState();
		void setScriptState(ScriptState _value);
		bool updateState(string &string_to_extract);
	private:
		ScriptState scriptState;
		queue<string> for_output;
};

*/

#endif // IO_H
