/*
 * Stellarium360 astronomy simulation and visualization
 *
 * Initial Author: Bernd "beko" Kosmahl from project 
 * Mechanized Assault and Exploration Reloaded Projectfile
 * Copyright (C) 2015 of Association Sirius
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Stellarium360 is a free open project of of LSS team
 * See the TRADEMARKS file for free open project usage requirements.
 *
 */
 
#include <iostream>
#include <time.h>
#include "log.hpp"
#include "cmutex.hpp"
#include "stellarium360.hpp"
#include <string>

//#include "settings.hpp"

//#define LogPath "stellarium360.log"
//#define ScriptLogPath "stellarium360-script.log"

/** errors */
#define EE "(EE): "
/** warnings */
#define WW "(WW): "
/** informations */
#define II "(II): "
/** debuginformations */
#define DD "(DD): "
/**mem error*/
#define MM "(MM): "
/** UNKNOWN error*/
#define UK "(UK): "


cLog::cLog():
	mutex()
{
	logfile = NULL;
	bScriptLogStarted = false;
	bFirstRun = true;
	isDebug=true;
	isScriptDebug=true;
}

bool cLog::open(int TYPE)
{
	//check if all log file name are initialized
	if ( bFirstRun )
	{
		time_t tTime = time ( NULL );
		tm * tmTime = localtime ( &tTime );
		char timestr[25];
		strftime( timestr, 21, "-%y.%m.%d-%H:%M.log", tmTime );
		std::string sTime = timestr;
		LogPath=      std::string(getenv("HOME"))+"/."+std::string(APP_LOWER_NAME)+"/log/stellarium360"+timestr;
	    ScriptLogPath=std::string(getenv("HOME"))+"/."+std::string(APP_LOWER_NAME)+"/log/script360"+timestr;
	    //printf("\n%s\n%s\n",LogPath.c_str(),ScriptLogPath.c_str());
		bFirstRun = false;
	}

	if ( logfile == NULL || ((TYPE == LOG_TYPE_SCRIPT_DEBUG || TYPE == LOG_TYPE_SCRIPT_WARNING || TYPE == LOG_TYPE_SCRIPT_ERROR || TYPE == LOG_TYPE_SCRIPT_INFO) && !bScriptLogStarted) )
	{
		if(TYPE == LOG_TYPE_SCRIPT_DEBUG || TYPE == LOG_TYPE_SCRIPT_WARNING || TYPE == LOG_TYPE_SCRIPT_ERROR || TYPE == LOG_TYPE_SCRIPT_INFO)
		{
			logfile = SDL_RWFromFile ( ScriptLogPath.c_str(),"w+t" ); //Start new network-log and write at beginning of file
			bScriptLogStarted = true;
		}
		else
		{
			logfile = SDL_RWFromFile ( LogPath.c_str(),"w+t" ); //Start new log and write at beginning of file
		}
	}
	else
	{
		if(TYPE == LOG_TYPE_SCRIPT_DEBUG || TYPE == LOG_TYPE_SCRIPT_WARNING || TYPE == LOG_TYPE_SCRIPT_ERROR || TYPE == LOG_TYPE_SCRIPT_INFO )
		{
			logfile = SDL_RWFromFile ( ScriptLogPath.c_str(),"a+t" ); //Reopen script-log and write at end of file
		}
		else
		{
			logfile = SDL_RWFromFile ( LogPath.c_str(),"a+t" ); //Reopen log and write at end of file
		}
	}

	int blocks; //sanity check - is file readable?
	char buf[256];

	if(logfile) //can access logfile
	{
		blocks=SDL_RWread ( logfile,buf,16,256/16 );
	}
	else
	{
		fprintf ( stderr,"(EE): Couldn't open file log!\n Please check file/directory permissions\n" );
		return false;
	}
	if ( blocks<0 )
	{
		fprintf ( stderr,"(EE): Couldn't read file log!\n Please check file/directory permissions\n" );

		if ( logfile != NULL ) return true;
		else return false;
	}
	else return true;

}

int cLog::write ( const char *str, int TYPE )
{
	return write ( std::string(str), TYPE );
}

int cLog::write ( std::string str, int TYPE )
{
	cMutex::Lock l(mutex);

	//~ if (TYPE == LOG_TYPE_DEBUG && !isDebug ) //in case debug is disabled we skip message //Todo: fix isDebug in app_setting
	//~ {
		//~ return 0;
	//~ }
	//~ if (TYPE == LOG_TYPE_SCRIPT_DEBUG && !isScriptDebug ) //in case script_debug is disabled we skip message //Todo: fix isScriptDebug in app_setting
	//~ {
		//~ return 0;
	//~ }

	if ( open(TYPE) )
	{
		switch ( TYPE ) //Attach log message type to tmp
		{
			case LOG_TYPE_UNKNOWN : str = str.insert ( 0 , UK ); break;
			case LOG_TYPE_SCRIPT_WARNING :
			case LOG_TYPE_WARNING : str = str.insert ( 0 , WW ); break;
			case LOG_TYPE_SCRIPT_ERROR :
			case LOG_TYPE_ERROR :   str = str.insert ( 0 , EE ); std::cout <<  str << std::endl; break;
			case LOG_TYPE_SCRIPT_DEBUG :
			case LOG_TYPE_DEBUG :   str = str.insert ( 0 , DD ); std::cout << str << std::endl; break;
			case LOG_TYPE_SCRIPT_INFO :
			case LOG_TYPE_INFO :    str = str.insert ( 0 , II ); break;
			case LOG_TYPE_MEM :	 str = str.insert ( 0 , MM ); break;
			default :				str = str.insert ( 0 , II );
		}
		str += "\n";
		return writeMessage ( str ); //add log message itself to tmp and send it for writing
	}
	else return -1;
}

int cLog::write ( const char *str)
{
	return write ( std::string ( str ) , LOG_TYPE_INFO );
}

int cLog::write ( std::string str)
{
	return write ( str , LOG_TYPE_INFO );
}

void cLog::mark()
{
	cMutex::Lock l(mutex);
	std::string str = "=================================================================";
	str += "\n";
	if ( open(-1) ) writeMessage ( str );
}

int cLog::writeMessage ( char *str )
{
	return writeMessage ( std::string ( str ) );
}

int cLog::writeMessage ( std::string str )
{
	if(logfile)
	{
		int const wrote = SDL_RWwrite(logfile, str.c_str(), 1, (int)str.length());
		if ( wrote<0 ) //sanity check - was file writable?
		{
			fprintf ( stderr,"Couldn't write to file log\nPlease check file/directory\nLog message was:\n%s", str.c_str() );
			return -1;
		}
		else close(); //after successful writing of all information we close log here and nowhere else!
		return 0;
	}
	else
	{
		fprintf ( stderr,"Couldn't write to file log\nPlease check file/directory\nLog message was:\n%s", str.c_str() );
	}
	return -1;
}


void cLog::close()
{
	SDL_RWclose ( logfile ); //function RWclose always returns 0 in SDL <= 1.2.9 - no sanity check possible
}
