/*
 * Stellarium360 astronomy simulation and visualization
 *
 * Initial Author: Bernd "beko" Kosmahl from project 
 * Mechanized Assault and Exploration Reloaded Projectfile
 * 
 * Copyright (C) 2014 of the LSS Team & Association Sirius
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Stellarium360 is a free open project of of LSS team
 * See the TRADEMARKS file for free open project usage requirements.
 *
 */
 
#ifndef LOG_H
#define LOG_H

#include <iostream>
#include <SDL2/SDL_rwops.h>
//#include "defines.hpp"
#include "cmutex.hpp"
#include "stellarium360.hpp"

#define LOG_TYPE_UNKNOWN 0
#define LOG_TYPE_WARNING 1
#define LOG_TYPE_ERROR 2
#define LOG_TYPE_DEBUG 3
#define LOG_TYPE_INFO 4
#define LOG_TYPE_MEM 5
#define LOG_TYPE_SCRIPT_DEBUG 6
#define LOG_TYPE_SCRIPT_WARNING 7
#define LOG_TYPE_SCRIPT_ERROR 8
#define LOG_TYPE_SCRIPT_INFO 9

/**
* Log class. Simple log class :-)
*
* @author Bernd "beko" Kosmahl
*/
class cLog
{
private:
	SDL_RWops *logfile;
	bool bScriptLogStarted;
	bool bFirstRun;
	bool isDebug;
	bool isScriptDebug;
	cMutex mutex;
	
	std::string LogPath;
	std::string ScriptLogPath;
	/**
	* Writes message finally to logfile
	*/
	int writeMessage( char *);
	int writeMessage( std::string );

	/**
	* Closes the logfile.
	*/
	void close();
	/**
	* Opens the Logfile.
	*
	* @return true on success
	*/
	bool open(int TYPE);
public:
	cLog();
	/**
	* Writes message with given type to logfile
	*
	* @param str Message for the log
	* @param TYPE Type for the log.<br>
	* 1 		== warning 	(WW):<br>
	* 2 		== error	(EE):<br>
	* 3		== debug	(DD):<br>
	* 4		== information	(II):<br>
	* 5		== memory prob. (MM):<br>
	* else		== information	(II):
	*
	* @return 0 on success
	*/
	int write( const char *str ,int TYPE );
	int write( std::string str , int TYPE );

	/**
	* Writes message with default type (II) to the logfile
	*
	* @param str Message for the log
	*
	* @return 0 on success
	*/
	int write( const char *str );
	int write( std::string str );

	enum LOG_TYPE
	{
		eLOG_TYPE_UNKNOWN = LOG_TYPE_UNKNOWN,
		eLOG_TYPE_WARNING = LOG_TYPE_WARNING,
		eLOG_TYPE_ERROR   = LOG_TYPE_ERROR,
		eLOG_TYPE_DEBUG   = LOG_TYPE_DEBUG,
		eLOG_TYPE_INFO    = LOG_TYPE_INFO,
		eLOG_TYPE_MEM	  = LOG_TYPE_MEM,
		eLOG_TYPE_SCRIPT_DEBUG   = LOG_TYPE_SCRIPT_DEBUG,
		eLOG_TYPE_SCRIPT_WARNING = LOG_TYPE_SCRIPT_WARNING,
		eLOG_TYPE_SCRIPT_ERROR   = LOG_TYPE_SCRIPT_ERROR,
		eLOG_TYPE_SCRIPT_INFO    = LOG_TYPE_SCRIPT_INFO
	};

	/**
	* Writes a marker into logfile - please use only veeeery few times!
	*/
	void mark();
	
	
	/**
	 * get private flag variable "isDebug"
	 */
	 bool getLogIsDebug () {
		 return isDebug;
	}
	
	/**
	 * set variable "isDebug" to a 
	 */
	
	void setLogIsDebug (bool a) {
		isDebug=a;
	}

	/**
	 * get private flag variable "isDebug"
	 */
	 bool getLogIsScriptDebug () {
		 return isScriptDebug;
	}
	
	/**
	 * set variable "isScriptDebug" to a 
	 */
	
	void setLogIsScriptDebug (bool a) {
		isScriptDebug=a;
	}
};


//do not include #include "log.hpp" in all .h 
#ifdef __main__
cLog Log;
#else
extern cLog Log;
#endif




#endif
