#include "stellarium360.hpp"
#include "system.hpp"
#include<SDL2/SDL.h>
//#include<GL/glew.h>
//#include<GL/gl.h>
//#include<GL/glu.h>
//#include<GL/glext.h>

#include "model3D.hpp"
#include <iostream>

/* ==== MODEL3D CLASS ===============================================*/

string model3D::model3DDir = "./";

//for static creation
model3D::model3D()
{
	inputfile="";
	is_ok = true;
	shapes.clear();
	materials.clear();
	err.clear();
	//printf("Empty model3D created\n");
}


//for pointer creation
model3D::model3D(const string filename)
{
	model3D(filename.c_str());
}

model3D::model3D(const char *filename)
{
	inputfile = model3DDir+filename+".obj";
	is_ok = true;
	shapes.clear();
	materials.clear();
	err.clear();
	//~ printf("Model name %s\n", inputfile.c_str());
}


model3D::~model3D()
{
//    glDeleteLists(liste,1);
}

void model3D::init()
{
	load_OBJ(inputfile.c_str());
}

void model3D::init(string filename) {
	inputfile = model3DDir+filename+".obj";
	load_OBJ(inputfile.c_str());
}

void model3D::load_OBJ(const char* filename)
{
    // Convert char array to string class
    //inputfile = filename;
    //printf("essai de chargement du fichier %s\n",filename);
	is_ok= true;
    err = tinyobj::LoadObj(shapes, materials, inputfile.c_str());

    //printf("LoadObj say: %s\n",err.c_str());

    if (!err.empty()) {
      //std::cerr << err << std::endl;
      printf( "\tError LoadObj %s\n",err.c_str() );
      printf( "\tError LoadObj\n");
      is_ok = false;
      return;
    }

    //~ std::cout << "# of shapes    : " << shapes.size() << std::endl;
    //~ std::cout << "# of materials : " << materials.size() << std::endl;


    for (size_t i = 0; i < shapes.size(); i++) {
      //~ printf("shape[%ld].name = %s\n", i, shapes[i].name.c_str());
      //~ printf("Size of shape[%ld].indices: %ld\n", i, shapes[i].mesh.indices.size());
      //~ printf("Size of shape[%ld].material_ids: %ld\n", i, shapes[i].mesh.material_ids.size());
      if ((shapes[i].mesh.indices.size() % 3) != 0) {
		printf("Indices error\n");
		is_ok = false;
		return;
		}
      //~ for (size_t f = 0; f < shapes[i].mesh.indices.size() / 3; f++) {
        //~ printf("  idx[%ld] = %d, %d, %d. mat_id = %d\n", f, shapes[i].mesh.indices[3*f+0], shapes[i].mesh.indices[3*f+1], shapes[i].mesh.indices[3*f+2], shapes[i].mesh.material_ids[f]);
      //~ }

      //~ printf("shape[%ld].vertices: %ld\n", i, shapes[i].mesh.positions.size());
      if ((shapes[i].mesh.positions.size() % 3) != 0) {
			printf("Vertices error\n");
			is_ok = false;
			return;
		}
      //~ for (size_t v = 0; v < shapes[i].mesh.positions.size() / 3; v++) {
        //printf("  v[%ld] = (%f, %f, %f)\n", v,
        //  shapes[i].mesh.positions[3*v+0],
        //  shapes[i].mesh.positions[3*v+1],
        //  shapes[i].mesh.positions[3*v+2]);*/
      //~ }
    }

    /*for (size_t i = 0; i < materials.size(); i++) {
      printf("material[%ld].name = %s\n", i, materials[i].name.c_str());
      printf("  material.Ka = (%f, %f ,%f)\n", materials[i].ambient[0], materials[i].ambient[1], materials[i].ambient[2]);
      printf("  material.Kd = (%f, %f ,%f)\n", materials[i].diffuse[0], materials[i].diffuse[1], materials[i].diffuse[2]);
      printf("  material.Ks = (%f, %f ,%f)\n", materials[i].specular[0], materials[i].specular[1], materials[i].specular[2]);
      printf("  material.Tr = (%f, %f ,%f)\n", materials[i].transmittance[0], materials[i].transmittance[1], materials[i].transmittance[2]);
      printf("  material.Ke = (%f, %f ,%f)\n", materials[i].emission[0], materials[i].emission[1], materials[i].emission[2]);
      printf("  material.Ns = %f\n", materials[i].shininess);
      printf("  material.Ni = %f\n", materials[i].ior);
      printf("  material.dissolve = %f\n", materials[i].dissolve);
      printf("  material.illum = %d\n", materials[i].illum);
      printf("  material.map_Ka = %s\n", materials[i].ambient_texname.c_str());
      printf("  material.map_Kd = %s\n", materials[i].diffuse_texname.c_str());
      printf("  material.map_Ks = %s\n", materials[i].specular_texname.c_str());
      printf("  material.map_Ns = %s\n", materials[i].normal_texname.c_str());
      std::map<std::string, std::string>::const_iterator it(materials[i].unknown_parameter.begin());
      std::map<std::string, std::string>::const_iterator itEnd(materials[i].unknown_parameter.end());
      for (; it != itEnd; it++) {
        //~ //printf("  material.%s = %s\n", it->first.c_str(), it->second.c_str());
      }
      printf("\n");
    }*/

    float x,y,z; // to compute the distance between the center and the farest point
    float tmpd,d=0.0; // temporary distance , distance
    float multiplicator;


    for (size_t i = 0; i < shapes.size(); i++) {
        for(size_t v = 0; v < shapes[i].mesh.positions.size() ; v+=3) {

            x=shapes[i].mesh.positions[v+0];
            y=shapes[i].mesh.positions[v+1];
            z=shapes[i].mesh.positions[v+2];

            tmpd=x*x+y*y+z*z;
            if(tmpd>d)
                d=tmpd;
        }
    }

    d=sqrt(d);
   // printf("Obj: radius %f\n",d);

    if(d!=0.0)
    {
        multiplicator = 1.0/d;

        for (size_t i = 0; i < shapes.size(); i++) {
            for(size_t v = 0; v < shapes[i].mesh.positions.size() ; v+=3) {

                shapes[i].mesh.positions[v+0]*=multiplicator;
                shapes[i].mesh.positions[v+1]*=multiplicator;
                shapes[i].mesh.positions[v+2]*=multiplicator;
            }
        }
    }
    else {
		printf("Obj: radius = 0\n");
		is_ok= false;
		return;
	}

	//printf("Model3D::load_Obj end\n");
	is_ok = true;
}


/* ==== MODEL3D_MGR CLASS ============================================*/


model3D_mgr::model3D_mgr(string _obj3DDir){
	obj3DDir=_obj3DDir;
}

model3D_mgr::~model3D_mgr(){
	list<obj3D* >::iterator it;
	for (it=obj3DList.begin(); it!=obj3DList.end(); ++it)
		delete *it;
	obj3DList.clear();
}

//~ bool model3D_mgr::addStellObj(string name)
//~ {
	//~ obj3D * tmp;
	//~ tmp=add_StellObj(name);
	//~ if (tmp==NULL)
		//~ return true;
	//~ else
		//~ return false;
//~ }

obj3D * model3D_mgr::addStellObj(string name)
{
	obj3D * tmp=NULL;
	tmp=get_name(name);
	if (tmp == NULL) {
		tmp=add_InternalObj(name);
		//chargement par défaut de la sphere sur un model au nom faux
		if (tmp == NULL) {
			tmp = get_name("Sphere"); //normalement le model3D de la sphere est chargée en mémoire par defaut

			//dernière verification, si le model3D de base n'existe pas, on stoppe le logiciel
			if (tmp == NULL) {
				printf("erreur de chargement du fichier 3D %s\n",name.c_str());
				exit(3);
			}
		}
		return tmp;
	}
	else
		return tmp;
}

obj3D * model3D_mgr::get_name(string _name)
{
	list<obj3D* >::iterator it;
	for (it=obj3DList.begin(); it!=obj3DList.end(); ++it) {
		if ((*it)->name == _name)
			return *it;
	}
	return NULL;
}

bool model3D_mgr::addInternalObj(string name)
{
	obj3D * tmp;
	tmp=add_InternalObj(name);
	if (tmp==NULL)
		return true;
	else
		return false;
}

obj3D * model3D_mgr::add_InternalObj(string name)
{
	string repertory=obj3DDir+"/"+name;
	if ( dir_exist(repertory) ) {
		if ( (file_exist(repertory+"/"+name+"_1L.obj")) && (file_exist(repertory+"/"+name+"_2M.obj")) && (file_exist(repertory+"/"+name+"_3H.obj")) )
		{
			obj3D *tmp = new obj3D;
			tmp->name= name;

			tmp->low    = new model3D();
			tmp->low->init(name+"/"+name+"_1L");

			tmp->medium = new model3D();
			tmp->medium->init(name+"/"+name+"_2M");

			tmp->high   = new model3D();
			tmp->high->init(name+"/"+name+"_3H");

			if (tmp->low->getOk() && tmp->medium->getOk() && tmp->high->getOk())  {
				//~ printf("correct\n");
				obj3DList.push_back(tmp);
				return tmp;
			}
			else {
				printf("erreur de chargement d'un model3D\n");
				//on détruit l'objet puisqu'il n'est pas complet
				delete tmp;
				return NULL;
			}
		}
		else {
			printf("le dossier du model3D n'existe pas\n");
			return NULL;
		}

		//pour que le compilateur ne rale pas
		return NULL;
	}
	else
	return NULL;
}
