#ifndef MODEL3D_HPP_INCLUDED
#define MODEL3D_HPP_INCLUDED

#include "s_texture.hpp"
#include <list>
#include "tiny_obj_loader.hpp"
/**
*   \class model3D
*
*   Cette classe permet de charger et affich un fichier .obj.
*   Pour charger un .obj, utiliser load_OBJ suivi du nom du fichier OBJ et du nom du fichier BMP qui sert de texture.
*   draw() permet ensuite de dessiner le modèle chargé.
*
*/

class model3D
{
    friend class Projector;

public:
    model3D();
    model3D(const char*filename);
    model3D(const string filename);
    ~model3D();

	static void set_model3DDir(const string& _model3DDir) {
		model3D::model3DDir = _model3DDir;
	}

	bool getOk() {
		return is_ok;
	}

	void init();
	void init(string filename);
    
private:
    std::string inputfile;
    std::vector<tinyobj::shape_t>    shapes;
    std::vector<tinyobj::material_t> materials;
    std::string err;

	bool is_ok; //say if the model is correctly initialised and operationnal
    void load_OBJ(const char*filename);
	static string model3DDir;

    GLuint liste; // Liste prête à l'emploi du modèle;

};


struct obj3D
{
	string name;
	model3D *low;
	model3D *medium;
	model3D *high;
};


class model3D_mgr
{
public:
	model3D_mgr(string _obj3DDir);
	~model3D_mgr();
	obj3D* addStellObj(string name);
	bool addInternalObj(string name);
private:
	obj3D* add_InternalObj(string name);
	obj3D* get_name(string name);
	list <obj3D*> obj3DList;
	string obj3DDir;
};

#endif // MODEL3D_HPP_INCLUDED



/*
* a utiliser comme ceci:
*
*   for (size_t i = 0; i < shapes.size(); i++) {
		glBegin(GL_TRIANGLES);
		for(size_t v = 0; v < shapes[i].mesh.indices.size() / 3; v++) {
			int i1=shapes[i].mesh.indices[3*v+0],
				i2=shapes[i].mesh.indices[3*v+1],
				i3=shapes[i].mesh.indices[3*v+2];
			glTexCoord2f(shapes[i].mesh.texcoords[2*i1+0],shapes[i].mesh.texcoords[2*i1+1]);
			glVertex3d(
			shapes[i].mesh.positions[3*i1+0],
			shapes[i].mesh.positions[3*i1+1],
			shapes[i].mesh.positions[3*i1+2]);
			glTexCoord2f(shapes[i].mesh.texcoords[2*i2+0],shapes[i].mesh.texcoords[2*i2+1]);
			glVertex3d(
			shapes[i].mesh.positions[3*i2+0],
			shapes[i].mesh.positions[3*i2+1],
			shapes[i].mesh.positions[3*i2+2]);
			glTexCoord2f(shapes[i].mesh.texcoords[2*i3+0],shapes[i].mesh.texcoords[2*i3+1]);
			glVertex3d(
			shapes[i].mesh.positions[3*i3+0],
			shapes[i].mesh.positions[3*i3+1],
			shapes[i].mesh.positions[3*i3+2]);
		}
		glEnd();
	}
*
*/
