#ifndef _OBJECT_TYPE_H_
#define _OBJECT_TYPE_H_

#include "intrusive_ptr.hpp"

#include <limits>
#include <string>
#include <cstring>
//#include "mboost.hpp"
#include "utility.hpp"

struct ObjectRecord {
	enum OBJECT_TYPE {
		OBJECT_UNINITIALIZED,
		OBJECT_STAR,
		OBJECT_ENTITY,
		OBJECT_NEBULA,
		OBJECT_CONSTELLATION,
//		OBJECT_TELESCOPE // inutile
	};

	ObjectRecord():
		englishName(NULL),
		shortName(NULL),
		nameI18(NULL),
		hpNumber(std::numeric_limits<unsigned int>::max()),
		type(OBJECT_UNINITIALIZED),
		mag(std::numeric_limits<float>::max()),
		distance(std::numeric_limits<float>::max()),
		units(NULL){

		//tmp = boost::lexical_cast<std::string>(type);
		tmp=Utility::intToString(type);
		strType = tmp.c_str();
	}
	ObjectRecord( const char* en, const char* loc, unsigned int hp, OBJECT_TYPE t, float m, float d ):
		// Star Constructor
		englishName(en),
		shortName(NULL),
		nameI18(loc),
		hpNumber(hp),
		type(t),
		mag(m),
		distance(d),
		units("LY"){

		//tmp = boost::lexical_cast<std::string>(type);
		tmp=Utility::intToString(type);
		strType = tmp.c_str();
	}
	ObjectRecord( const char* en, const char* loc, OBJECT_TYPE t, float m ):
		// Nebula Constructor
		englishName(en),
		shortName(NULL),
		nameI18(loc),
		hpNumber(std::numeric_limits<unsigned int>::max()),
		type(t),
		mag(m),
		distance(std::numeric_limits<float>::max()),
		units(NULL){

		//tmp = boost::lexical_cast<std::string>(type);
		tmp=Utility::intToString(type);
		strType = tmp.c_str();
	}
	ObjectRecord( const char* en, const char* loc, const char* abv, OBJECT_TYPE t ):
		// Constellation Constructor
		englishName(en),
		shortName(abv),
		nameI18(loc),
		hpNumber(std::numeric_limits<unsigned int>::max()),
		type(t),
		mag(std::numeric_limits<float>::max()),
		distance(std::numeric_limits<float>::max()),
		units(NULL){

		//tmp = boost::lexical_cast<std::string>(type);
		tmp=Utility::intToString(type);
		strType = tmp.c_str();
	}
	ObjectRecord( const char* en, const char* loc, OBJECT_TYPE t ):
		// Planet constructor
		englishName(en),
		shortName(NULL),
		nameI18(loc),
		hpNumber(std::numeric_limits<unsigned int>::max()),
		type(t),
		mag(std::numeric_limits<float>::max()),
		distance(std::numeric_limits<float>::max()),
		units("AU"){
			
		//tmp = boost::lexical_cast<std::string>(type);
		tmp=Utility::intToString(type);
		strType = tmp.c_str();
	}

	char const* englishName;
	char const* shortName;
	char const* nameI18;
	char const* strType;
	unsigned int hpNumber;
	unsigned int type;
	float mag;
	float distance;
	char const* units;
	std::string tmp;
};


//! Intrusive pointer used to manage StelObject with smart pointers
class ObjectBase;
typedef boost::intrusive_ptr<ObjectBase> ObjectBaseP;

#endif
