/*
 * Stellarium360 astronomy simulation and visualization
 *
 * Copyright (C) 2002 Fabien Chereau
 * Copyright (C) 2009 Digitalis Education Solutions, Inc.
 * Copyright (C) 2014 of the LSS Team & Association Sirius
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Stellarium360 is a free open project of of LSS team
 * See the TRADEMARKS file for free open project usage requirements.
 *
 */

#ifndef _RING_H_
#define _RING_H_

#include "program_object.hpp"
#include "object_base.hpp"
//#include "stellarium360.hpp"
#include "utility.hpp"
#include "s_font.hpp"
#include "tone_reproductor.hpp"
#include "vecmath.hpp"
#include "callbacks.hpp"
#include "fader.hpp"
//#include "translator.hpp"
#include "orbit.hpp"
//#include "model3D.hpp"

// Class to manage rings for planets like saturn
class Ring
{
public:
	Ring(double radius_min,double radius_max,const string &texname, const string &path);
	~Ring(void);
	void draw(const Projector* prj,const Mat4d& mat,double screen_sz,Vec3f& lightDirection,Vec3f& planetPosition, float planetRadius);
	double get_size(void) const {
		return radius_max;
	}
	double get_inner_radius(void) const {
		return radius_min;
	}
	const s_texture *get_tex(void) const {
		return tex;
	}
private:
	const double radius_min;
	const double radius_max;
	const s_texture *tex;

	// Shaders
	ProgramObject programObject;
	bool shaderEnable;

};


#endif // _RING_H_
