/*
 * Stellarium360 astronomy simulation and visualization
 *
 * Copyright (C) 2014 Association Sirius
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Stellarium360 is a free open project of of LSS team
 * See the TRADEMARKS file for free open project usage requirements.
 *
 */

#include <iostream>
//#include <ostream>
#include <string>
#include <fstream>
#include <vector>
#include "script.hpp"
#include <cstddef>
#include "log.hpp"

using namespace std;

template <typename Container>
Container& split( Container& result, const typename Container::value_type& s, const typename Container::value_type& delimiters)
{
  result.clear();
  size_t current;
  size_t next = -1;
  do
  {
      next = s.find_first_not_of( delimiters, next + 1 );
      if (next == Container::value_type::npos) break;
      next -= 1;

    current = next + 1;
    next = s.find_first_of( delimiters, current );
    result.push_back( s.substr( current, next - current ) );
  }
  while (next != Container::value_type::npos);
  return result;
}


void print( vector <string> & v )
{
  for (size_t n = 0; n < v.size(); n++)
    Log.write(v[n],  cLog::eLOG_TYPE_SCRIPT_DEBUG);
}

Token::Token(string s, string p)
{
		elmt="";
		path="";
		pNext=NULL;
		elmt=s;
		path=p;
}

Token::~Token()
{
		if (pNext != NULL) delete pNext;
}

void Token::printtoken()
{
		Log.write("Token script : " + path + " : " + elmt,  cLog::eLOG_TYPE_SCRIPT_DEBUG);
}

string Token::getToken()
{
		return elmt;
}

string Token::getTokenPath()
{
		return path;
}

Script::Script()
{
	//cout << "Script->init()" << endl;
	pFirst=NULL;
	pLast=NULL;
	p2First=NULL;
	p2Last=NULL;
	path="";
}

Script::~Script()
{
	if (pFirst != NULL) delete pFirst;
}

void Script::printScript()
{
	Token *pMove=pFirst;	
	Log.write("Printing script: --BEGIN",  cLog::eLOG_TYPE_SCRIPT_DEBUG);
	while (pMove !=NULL)
	{
		Log.write("   " + pMove->getToken(),  cLog::eLOG_TYPE_SCRIPT_DEBUG);
		pMove=pMove->pNext;
	}	
	Log.write("Printing script: --END",  cLog::eLOG_TYPE_SCRIPT_DEBUG);
}

void Script::clean()
{
	if (pFirst != NULL) delete pFirst;
	pFirst = NULL;
	pLast = NULL;
	p2First = NULL;
	p2Last = NULL;
	path="";
}

int Script::load(string script_file, string script_path )
{
	if (pFirst == NULL)
		{
		//cout << "loading script "<< script_file << " when no script is playing" << endl;
		return loadI(script_file, script_path ,0);
		}
	else
		{
		//cout << "loading script " << script_file << " when script is playing" << endl;
		return loadI(script_file, script_path ,1);
		}
}

int Script::loadI(string script_file, string script_path , int wp)
{
	input_file = new ifstream(script_file.c_str());

	if (! input_file->is_open()) {
		Log.write("Unable to open script: " + script_file,  cLog::eLOG_TYPE_SCRIPT_ERROR);
		return 0;
	}
	path = script_path;
	vector <string> fields;
	//~ bool is_comment=false;
	//~ bool is_repeat=false;
	//~ int nbr_repeat=0;
	
	bool is_script_empty=true;
	string line;
	Token *token=NULL;
	while (! input_file->eof() )
	{
		getline(*input_file,line);
	
		if ( line[0] != '#' && line[0] != 0 && line[0] != '\r' && line[0] != '\n') 
		{
			//~ split( fields, line, " ");
			//~ print(fields);
			//~ cout << fields.size() << " fields.\n";

			//cout << "[script.cpp => Line is: " << line << "]"<< endl;
			token=new Token(line, path);
			is_script_empty=false;
			if (wp==0)
				add(token);
			else
				add2(token);
		}
	}
	input_file->close();
	if ((! is_script_empty) && (wp==1))
	{
		p2Last->pNext=pFirst;
		pFirst=p2First;
		p2Last=NULL;
		p2First=NULL;
	}
	//printScript();
	return 1;
}

void Script::add(Token *token)
{
	if (pFirst == NULL)
	{
		pFirst=token;
		pLast=token;
	}
	else
	{
		pLast->pNext=token;
		pLast=pLast->pNext;
	}
}

void Script::add2(Token *token)
{
	if (p2First == NULL)
	{
		p2First=token;
		p2Last=token;
	}
	else
	{
		p2Last->pNext=token;
		p2Last=p2Last->pNext;
	}
}

int Script::getFirst(string &command, string &dataDir)
{
	Token *pMove;
	if (pFirst == NULL)
	{
		Log.write("End of script",  cLog::eLOG_TYPE_SCRIPT_INFO);
		command="script action end";
		return 0;
	}
	else
	{
		pMove=pFirst;
		//pFirst->printtoken();
		dataDir=pFirst->getTokenPath();
		command=pFirst->getToken();
		if (command=="script action end" && (pFirst->pNext != NULL))
		{
			Log.write("End of script  detected but not at end of the execution stack", cLog::eLOG_TYPE_SCRIPT_WARNING);
		}
		//cout << " script.cpp : " << command << endl;
		pFirst=pFirst->pNext;
		pMove->pNext=NULL;
		delete pMove;
		return 1;
	}
}

int Script::next_command(string &command)
{
	Log.write("Deprecied Fonction: script->next_command()", cLog::eLOG_TYPE_SCRIPT_WARNING);
	//~ command = getFirst();
	//~ if (command != "")
	return 0;
		//~ else
	//~ return 1;
}	


string Script::get_path()
{
	return path;
}	
