#include "shader.hpp"
#include <iostream>
#include <string.h>

using namespace std;

string shaderProgram::shaderDir = "./";

void shaderProgram::LoadSource(const char *filename,GLint*numberOfLines,GLchar***code,GLint**linesLengths)
{
    //char *src = NULL;   // source code of the shader
    FILE *fp = NULL;    // file
    long size;          // size of the file
    long i;             // iterator
    long j;             // iterator
    char c;             // temp
    (*numberOfLines)=1;   // lines counter
    //*linesLengths;  // need to be deleted after glShaderSource

    //cout<<"name: "<<filename<<endl;

    fp = fopen(filename, "r");
    if(fp == NULL){
        fprintf(stderr, "Unable to open '%s'\n", filename);
        exit(EXIT_FAILURE);
    }
    // First: get the size of the file.
    fseek(fp, 0, SEEK_END);
    size = ftell(fp);
    rewind(fp);// go back to the begin of the file
    //cout<<"size: "<<size<<endl;


    // Second: count the number of lines.
    for(i=0; i<size; i++)
    if(fgetc(fp)=='\n')
        (*numberOfLines)++;
    rewind(fp);
    fseek(fp,0,SEEK_SET);
    //cout<<"number of lines: "<<*numberOfLines<<endl;


    // Third: count the number of characters per line.
    int currentLine=0; // iterator
    (*linesLengths)=new GLint[(*numberOfLines)];
    for(i=0;i<(*numberOfLines);i++) // Initialisation
        (*linesLengths)[i]=0;
    for(i=0; i<size; i++){
        (*linesLengths)[currentLine]++;
        if(fgetc(fp)=='\n')
        {
            //cout<<"line["<<currentLine<<"] = "<<(*linesLengths)[currentLine]<<endl;
            currentLine++;
        }
    }
    rewind(fp);

    //fourth: fill the source code.
    *code=new GLchar*[(*numberOfLines)];
    for(i=0;i<(*numberOfLines);i++)
        (*code)[i]=new GLchar[(*linesLengths)[i]];

    for(i=0;i<(*numberOfLines);i++)
    for(j=0;j<(*linesLengths)[i];j++)
    {
        c=fgetc(fp);
        (*code)[i][j]=c;
    }

    fclose(fp);
}

void shaderProgram::debugShader(GLuint shader)
{
    char* log;
    GLsizei logsize = 0;
    GLint compile_status = GL_TRUE;
    glGetShaderiv(shader, GL_COMPILE_STATUS, &compile_status);
    if(compile_status != GL_TRUE)
    {
        // on recupere la taille du message d'erreur
        glGetShaderiv(shader, GL_INFO_LOG_LENGTH, &logsize);

        // on alloue un espace memoire dans lequel OpenGL ecrira le message
        log = (char*)malloc(logsize + 1);
        if(log == NULL){
            cout<<"impossible d'allouer de la memoire !\n"<<endl;
            exit(EXIT_FAILURE);
        }
        memset(log, '\0', logsize + 1);
        glGetShaderInfoLog(shader, logsize, &logsize, log);
        cout<<"impossible de compiler le shader :"<<endl<<log<<endl;
        free(log);
        glDeleteShader(shader);
        exit(EXIT_FAILURE);
    }
}

GLuint shaderProgram::makeShader(GLenum ShaderType, GLint numberOfLines,const GLchar**code,const GLint*linesLengths)
{
    GLuint shader = glCreateShader(ShaderType);
    if(shader == 0){
        cout<< "impossible de creer le shader:"<<endl<<"glCreateShader failed."<<endl;
        exit(EXIT_FAILURE);
    }
    glShaderSource(shader, numberOfLines, code, linesLengths);
    glCompileShader(shader);
    debugShader(shader);
    return shader;
}

GLuint shaderProgram::makeShader(string str,GLenum ShaderType)
{
    GLchar**code;
    int*linesLengths;
    int numberOfLines;

	str= shaderDir + str;
    //int i,j;
    LoadSource(str.c_str(),&numberOfLines,&code,&linesLengths);

    // Affichage
    //cout<<"number of lines: "<<numberOfLines<<endl;
    //for(i=0;i<numberOfLines;i++){
    //    /out<<"size of line "<<i<<": "<<linesLengths[i]<<endl;
    //    for(j=0;j<linesLengths[i];j++)
    //        cout<<code[i][j];
    //    cout<<endl;
    //}

    GLuint shader = glCreateShader(ShaderType);
    if(shader == 0){
        fprintf(stderr, "impossible de creer le shader\n");
        exit(EXIT_FAILURE);
    }
    glShaderSource(shader,numberOfLines,(const GLchar**)code,linesLengths);
    delete code;
    delete linesLengths;
    code=NULL;
    glCompileShader(shader);
    debugShader(shader);


    delete code;
    return shader;
}

void shaderProgram::printInformations(GLuint program)
{
    GLint numUniforms =0;
    glGetProgramInterfaceiv(program,GL_UNIFORM,GL_ACTIVE_RESOURCES,&numUniforms);
    GLenum properties[]={GL_NAME_LENGTH, GL_TYPE, GL_LOCATION, GL_BLOCK_INDEX};
    printf("Program %d active uniforms:\n",program);
    for(int i=0;i<numUniforms;i++)
    {
        GLint results[4];
        glGetProgramResourceiv(program, GL_UNIFORM, i, 4, properties, 4, NULL, results);
        if(results[3]!=-1)
            continue; // skip uniforms in blocks
        GLint nameBuffSize= results[0]+1;
        char*name=new char[nameBuffSize];
        glGetProgramResourceName(program,GL_UNIFORM, i, nameBuffSize, NULL, name);
        switch(results[1])
        {
            case GL_INT:{
                int ivalue=0;
                glGetUniformiv(program,results[2],&ivalue);
                printf("Location %-5d: %s %d\n",results[2],name,ivalue);}
            break;
            case GL_FLOAT:{
                float fvalue=0;
                glGetUniformfv(program,results[2],&fvalue);
                printf("Location %-5d: %s %f\n",results[2],name,fvalue);}
            break;
            default:
                printf("Location %-5d: %s  type ID: %d\n",results[2],name,results[1]);
            break;

        }
        delete [] name;
    }
}

/*
GLint getUniformLocation(GLuint program,char*name)
{
    GLint result = glGetUniformLocation(program,name);
    if(name==-1)
        cout<<"Failed to set "<<name<<" in the "<<program<<" program object"<<endl
            <<"This variable is not used or his name is incorrect. Please check."<<endl
    return result;
}*/

shaderProgram::shaderProgram(GLuint vs,GLuint fs)
{
    init(vs,fs);
}

shaderProgram::~shaderProgram()
{
    glDetachShader(program,fshader);
    glDetachShader(program,vshader);
    glDeleteShader(fshader);
    glDeleteShader(vshader);
    glDeleteProgram(program);
}

void shaderProgram::init(GLuint vs,GLuint fs)
{
    vshader=vs;
    fshader=fs;
    program=glCreateProgram();
    glAttachShader(program,vshader);
    glAttachShader(program,fshader);
    glLinkProgram(program);
}

void shaderProgram::use()
{
    glUseProgram(program);
}






